/*////////////////////////////////////////////////////////////////////////////
    PROJECT:    Trailer Remote
    COPYRIGHT    2010 Dean J. Boman

    FILE:       Main.c
    AUTHOR:     Dean J. Boman

    OVERVIEW:
        Implementation of the main routine for the trailer vibration detection
		system.

    REVISION HISTORY:

    Sept. 13, 2010	Initial development based on the APRS code base
	Nov. 22, 2010	Add Battery monitor interface
	Jan. 7, 2013	Update for new trailer (add 2 additional wheel inputs, remove battery function)

*/////////////////////////////////////////////////////////////////////////////

#define BAUD_RATE       (19200)     // bps

// These headers must be included for required defs.

#include <p18cxxx.h>		//This brings in #include <p18f2620.h>

#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <delays.h>
#include "..\Include\Compiler.h"
#include "..\Include\typedef.h"


#define VERSION "3.0 Jan. 7, 2013   \r\n"

#define TIMERDELTA 1000				//Timer1 tick rate is 1 usec so generate 1 msec interrupt


//Determine SPBRG value for given baud rate
#define SPBRG_VAL   ( ((INSTR_FREQ/BAUD_RATE)/16) - 1)

#if(SPBRG_VAL > 255)
    #error "Calculated SPBRG value is out of range for current CLOCK_FREQ."
#endif


//Set configuration fuses
#pragma config OSC=INTIO67, FCMEN=OFF, IESO=OFF, PWRT=ON, WDT=ON, WDTPS = 256, LVP=OFF, DEBUG=OFF, PBADEN=OFF
//Note the watchdog is set for a 1 sec time out


//Prototypes for local functions
static void InitializeBoard(void);
uint16 sysCRC16 (uint8 *buffer, uint16 length, uint16 crc);
int16 TempRead (uint8 Address);
void VibSensorCal(void);

boolean TxFlag;
boolean StartUpFlag;

uint8 OutputBuffer[100];
uint8 StartUpCounter;

uint16 TimerCompare;
uint16 HalfSecCounter;
uint16 ADC_data;
uint16 RightFrontCal;
uint16 LeftFrontCal;
uint16 RightRearCal;
uint16 LeftRearCal;
uint16 CRC_value;

int16 RightFrontTemperature;
int16 LeftFrontTemperature;
int16 RightRearTemperature;
int16 LeftRearTemperature;

int16 RightFrontVibration;
int16 LeftFrontVibration;
int16 RightRearVibration;
int16 LeftRearVibration;

int16 RightFrontMaxPosVib;
int16 RightFrontMaxNegVib;
int16 RightFrontMaxVib;
int16 LeftFrontMaxPosVib;
int16 LeftFrontMaxNegVib;
int16 LeftFrontMaxVib;
int16 RightRearMaxPosVib;
int16 RightRearMaxNegVib;
int16 RightRearMaxVib;
int16 LeftRearMaxPosVib;
int16 LeftRearMaxNegVib;
int16 LeftRearMaxVib;

//Interrupt service routine for all interrupts.  
#pragma interrupt HighISR

void HighISR(void)
{
    //Determine if interrupt was caused by a CCP1 compare.  CCP1 interrupts occur
	//every 1 msec and are used to basic time keeping
	if(PIR1bits.CCP1IF)
    {
		//We have had a 1 msec interrupt
		HalfSecCounter++;
		if(HalfSecCounter >= 500)
		{
			TxFlag = TRUE;			//Time to send serial data to the display unit
			HalfSecCounter = 0;

			//Implement 10 second delay to allow things to settle out before doing sensor calibration
			if(StartUpFlag == TRUE)
			{
				StartUpCounter++;
				if(StartUpCounter == 20)
					StartUpFlag = FALSE;
			}
		}

		//Set the compare register for the next interval.  TIMERDELTA which is 1000 represents
		//1 msec on Timer1 which increments once every microsecond
		TimerCompare += TIMERDELTA;
		CCPR1H = (TimerCompare >> 8);
		CCPR1L = (TimerCompare & 0x00ff);

		PIR1bits.CCP1IF = 0;
	}
}


#pragma interruptlow LowISR

void LowISR(void)
{

}

//Define the interrupt vectors for both low and high interrupts
#pragma code highVector=0x08

void HighVector (void)
{
    _asm goto HighISR _endasm
}

#pragma code						//Return to default code section

#pragma code lowVector=0x18

void LowVector (void)
{
    _asm goto LowISR _endasm
}

#pragma code						//Return to default code section


// Main application entry point.

void main(void)
{
	TxFlag = FALSE;
	StartUpFlag = TRUE;
	StartUpCounter = 0;
	TimerCompare = 0;
	HalfSecCounter = 0;
	
	RightFrontVibration = 0;
	LeftFrontVibration = 0;
	RightRearVibration = 0;
	LeftRearVibration = 0;
	
	RightFrontMaxPosVib = 0;
	RightFrontMaxNegVib = 0;
	RightFrontMaxVib = 0;
	LeftFrontMaxPosVib = 0;
	LeftFrontMaxNegVib = 0;
	LeftFrontMaxVib = 0;
	RightRearMaxPosVib = 0;
	RightRearMaxNegVib = 0;
	RightRearMaxVib = 0;
	LeftRearMaxPosVib = 0;
	LeftRearMaxNegVib = 0;
	LeftRearMaxVib = 0;
	
	RightFrontCal = 0;
	LeftFrontCal = 0;
	RightRearCal = 0;
	LeftRearCal = 0;
	
	RightFrontTemperature = 0;
	LeftFrontTemperature = 0;
	RightRearTemperature = 0;
	LeftRearTemperature = 0;
	
	OutputBuffer[0] = 0;
	CRC_value = 0;

	//Sets the IO bits, ADC, and USART
    InitializeBoard();

	//Display current configuration info
	printf("\r\r\r");
	printf(VERSION);

	printf("We are now starting the main application loop\r\n");

	//Read vibration sensors to deterine earth acceleration level after a 10 second delay.
	//We need to read them before we start moving
	while(StartUpFlag == TRUE)
	{
		//Kick the dog while we are waiting for the time out
		_asm CLRWDT _endasm	
	}	
	VibSensorCal();

	printf("Sensor Cal values> %d, %d, %d, %d\r",RightFrontCal, LeftFrontCal, RightRearCal, LeftRearCal);

	//We now enter into an infinite loop.  Read the vibration sensors as fast as we can, recording the 
	//max/min values.  Send the data to the display unit twice per second.
    while(1)
    {
		//Read ADC to get current vibration data from each sensor and determine the max
		//level for the current measurement interval
		//Measurement of the hardware show that we pass this loop every 108 usec

		//Read the Right front vibration data
		ADCON0 = 0b00000001;		//Select the channel
		ADCON0bits.GO_DONE = 1;		//Start the conversion

		while(ADCON0bits.GO_DONE == 1);		//Wait until the conversion is complete

		ADC_data = ((uint16)ADRESH << 8) + ((uint16)ADRESL);

		RightFrontVibration = (int16)ADC_data - (int16)RightFrontCal;	//Upward acc is positive

		//Determine if the current vibration measurement is larger then encountered
		//during the current measurement interval.
		if(RightFrontVibration > 0)
		{
			if(RightFrontVibration > RightFrontMaxPosVib)
				RightFrontMaxPosVib = RightFrontVibration;
		}
		else
		{
			if(RightFrontVibration < RightFrontMaxNegVib)
				RightFrontMaxNegVib = RightFrontVibration;
		}
		
		//Read the Left front vibration data
		ADCON0 = 0b000000101;		//Select the channel
		ADCON0bits.GO_DONE = 1;		//Start the conversion

		while(ADCON0bits.GO_DONE == 1);		//Wait until the conversion is complete

		ADC_data = ((uint16)ADRESH << 8) + ((uint16)ADRESL);

		LeftFrontVibration = (int16)ADC_data - (int16)LeftFrontCal;	//Upward acc is positive

		//Determine if the current vibration measurement is larger then encountered
		//during the current measurement interval.
		if(LeftFrontVibration > 0)
		{
			if(LeftFrontVibration > LeftFrontMaxPosVib)
				LeftFrontMaxPosVib = LeftFrontVibration;
		}
		else
		{
			if(LeftFrontVibration < LeftFrontMaxNegVib)
				LeftFrontMaxNegVib = LeftFrontVibration;
		}


		//Read the Right rear vibration data
		ADCON0 = 0b00001001;		//Select the channel
		ADCON0bits.GO_DONE = 1;		//Start the conversion

		while(ADCON0bits.GO_DONE == 1);		//Wait until the conversion is complete

		ADC_data = ((uint16)ADRESH << 8) + ((uint16)ADRESL);

		RightRearVibration = (int16)ADC_data - (int16)RightRearCal;	//Upward acc is positive

		//Determine if the current vibration measurement is larger then encountered
		//during the current measurement interval.
		if(RightRearVibration > 0)
		{
			if(RightRearVibration > RightRearMaxPosVib)
				RightRearMaxPosVib = RightRearVibration;
		}
		else
		{
			if(RightRearVibration < RightRearMaxNegVib)
				RightRearMaxNegVib = RightRearVibration;
		}
	
		//Read the Left rear vibration data
		ADCON0 = 0b00001101;		//Select the channel
		ADCON0bits.GO_DONE = 1;		//Start the conversion

		while(ADCON0bits.GO_DONE == 1);		//Wait until the conversion is complete

		ADC_data = ((uint16)ADRESH << 8) + ((uint16)ADRESL);

		LeftRearVibration = (int16)ADC_data - (int16)LeftRearCal;	//Upward acc is positive

		//Determine if the current vibration measurement is larger then encountered
		//during the current measurement interval.
		if(LeftRearVibration > 0)
		{
			if(LeftRearVibration > LeftRearMaxPosVib)
				LeftRearMaxPosVib = LeftRearVibration;
		}
		else
		{
			if(LeftRearVibration < LeftRearMaxNegVib)
				LeftRearMaxNegVib = LeftRearVibration;
		}	
		
		//The ISR will set the TxFlag true once per second to indicate that it is time to send
		//the current measurement data to the display unit
		if(TxFlag == TRUE)
		{
			//Time to send data to the display unit
			//Read the temperature sensors
			RightFrontTemperature = TempRead(0);
			LeftFrontTemperature = TempRead(1);			
			RightRearTemperature = TempRead(2);
			LeftRearTemperature = TempRead(3);	
			
			//Determine the maximum vibration level (either positive or negative) in the current interval)

			if(RightFrontMaxPosVib > -RightFrontMaxNegVib)
				RightFrontMaxVib = RightFrontMaxPosVib;
			else
				RightFrontMaxVib = -RightFrontMaxNegVib;

			if(LeftFrontMaxPosVib > -LeftFrontMaxNegVib)
				LeftFrontMaxVib = LeftFrontMaxPosVib;
			else
				LeftFrontMaxVib = -LeftFrontMaxNegVib;

			if(RightRearMaxPosVib > -RightRearMaxNegVib)
				RightRearMaxVib = RightRearMaxPosVib;
			else
				RightRearMaxVib = -RightRearMaxNegVib;
				
			if(LeftRearMaxPosVib > -LeftRearMaxNegVib)
				LeftRearMaxVib = LeftRearMaxPosVib;
			else
				LeftRearMaxVib = -LeftRearMaxNegVib;

			//Build the output message
			sprintf(OutputBuffer,"Data>%d,%d,%d,%d,%d,%d,%d,%d",RightFrontMaxVib, LeftFrontMaxVib, 
				RightRearMaxVib, LeftRearMaxVib, RightFrontTemperature, LeftFrontTemperature, 
				RightRearTemperature, LeftRearTemperature);

			//Calculate the CRC of the message
			CRC_value = sysCRC16(OutputBuffer, strlen(OutputBuffer), 0x12);
	
			//Send the message
			printf("%s,%u\r",OutputBuffer,CRC_value);

			//Reset the vibration max registers
			RightFrontMaxPosVib = 0;
			RightFrontMaxNegVib = 0;
			LeftFrontMaxPosVib = 0;
			LeftFrontMaxNegVib = 0;
			RightRearMaxPosVib = 0;
			RightRearMaxNegVib = 0;
			LeftRearMaxPosVib = 0;
			LeftRearMaxNegVib = 0;
			
			ONE_SEC_LED ^= 1;		//Flash the LED to show that we are alive
			
			TxFlag = FALSE;			//Transmission is complete to clear the flag

		}		//if(TxFlag == TRUE)
		

		//Kick the dog
		_asm CLRWDT _endasm

    }	//while(1)
}		//void main(void)



/**
	Initialize the board by configuring the I/O ports, setting up the ADC, configuring the 
	UART, and enabling the interrupts.

  	@return void
*/
static void InitializeBoard(void)
{
	//The internal oscillator set to 8 MHz with PLL at x4
	OSCCONbits.IRCF0 = 1;
	OSCCONbits.IRCF1 = 1;
	OSCCONbits.IRCF2 = 1;

	OSCTUNEbits.PLLEN = 1;

	//Disable all interrupts until they are configured
    INTCONbits.GIEH = 0;	//Disable all interrupts
    INTCONbits.GIEL = 0;	//Perpheral interrupts disabled

	//Configure the ADC
	ADCON0 = 0b00000001;		//ADC ON
	ADCON1 = 0b00001011;		//AN0-AN3 as analog, 0 to 5 V reference
	ADCON2 = 0b10111010;		//Rigth justify, 20 TAD delay, Fosc/32 clock 

	//Configure timer 1 to generate a 1 msec interrupt in conjunction with
	//CPP1.  /8 prescale.  Timer clock is 1 usec (32/4/8)
    T1CONbits.RD16 = 1;
    T1CONbits.T1RUN = 0;
    T1CONbits.T1CKPS1 = 1;
    T1CONbits.T1CKPS0 = 1;
    T1CONbits.T1OSCEN = 0;
    T1CONbits.T1SYNC = 0;
    T1CONbits.TMR1CS = 0;
    T1CONbits.TMR1ON = 0;		//keep timer off for now

    TMR0H = 0;
    TMR0L = 0;

	//Set compare module to generate interrupt on compare and to use timer1 as the
	//clock source
    CCP1CONbits.CCP1M3 = 1;
    CCP1CONbits.CCP1M2 = 0;
    CCP1CONbits.CCP1M1 = 1;
    CCP1CONbits.CCP1M0 = 0;
	T3CONbits.T3CCP1 = 0;
	T3CONbits.T3CCP2 = 0;

	PIE1bits.CCP1IE = 1;		//Enable interrupts from CCP1
	PIR1bits.CCP1IF = 0;		//Clear interrupt flag

	//Set the compare register for the first interval
	TimerCompare = TIMERDELTA;
	CCPR1H = (TimerCompare >> 8);
	CCPR1L = (TimerCompare & 0x00ff);


	//Configure USART
	TRISCbits.TRISC6 = 0;		//USART Tx pin
	TRISCbits.TRISC7 = 1;		//USART Rx pin
    TXSTA = 0b00100000;			//Tx enable, Asyn mode, 8 bits Tx, low baud rate
    RCSTA = 0b10010000;			//Serial port enable, 8 bits Rx, Rx enable
    SPBRG = SPBRG_VAL;

	//Note:  The manual indicates that the UART automatically configures
	//the I/O pin for input when the UART receiver is selected but 
	//this does not look to be the case so we set the pin as an input 
	//to prevent contention.
	TRISC = TRISC | 0b10000000;

	ONE_SEC_LED_TRIS = 0;	//Configure the 1 second LED for output
	ONE_SEC_LED = 1;

	//Deselect all temperature sensors
	RIGHT_REAR_CS = 1;
	LEFT_REAR_CS = 1;
	RIGHT_FRONT_CS = 1;
	LEFT_FRONT_CS = 1;

	//Configure the CS for the temperature sensors
	RIGHT_REAR_CS_TRIS = 0;
	LEFT_REAR_CS_TRIS = 0;
	RIGHT_FRONT_CS_TRIS = 0;
	LEFT_FRONT_CS_TRIS = 0;

	//Configure the SPI port pins
	SCL_TRIS = 0;
	SDO_TRIS = 1;
	//SDO_TRIS = 0;
	SDI_TRIS = 1;

	//Configure the SPI port operation
	SSPSTAT = 0b01000000;		//Sample read in middle of data, write on falling edge
	SSPCON1 = 0b00100010;		//Low idle clock state, clock rate 500 KHz, SPI enabled

	T1CONbits.TMR1ON = 1;		//Start the timer
	PIE1bits.RCIE = 1;			//Enable serial interrupts

	// Enable Interrupts
    INTCONbits.GIEH = 1;		//High priority interrupts enabled
	INTCONbits.GIEL = 1;		//Perpheral interrupts enabled
}






/**
     Calculate the CRC-16 CCITT of <b>buffer</b> that is <b>length</b> bytes long.

     @param buffer Pointer to data buffer.
     @param length number of bytes in data buffer
	 @param crc starting value

     @return CRC-16 of buffer[0 .. length-1]
*/
uint16 sysCRC16 (uint8 *buffer, uint16 length, uint16 crc)
{
	uint16 i, bit, value;

	for (i = 0; i < length; ++i) {
		value = buffer[i];

		for (bit = 0; bit < 8; ++bit) {
			crc ^= (value & 0x01);
			crc = ( crc & 0x01 ) ? ( crc >> 1 ) ^ 0x8408 : ( crc >> 1 );
			value = value >> 1;
		}
	}		// End for (i = 0; i < length; ++i)

   return crc ^ 0xffff;
}




/**
     Reads the temperature from the selected temperature sensor.

     @param Address -  0 = Right front, 1 = Left front, 2 = Right rear, 3 = Left rear
     

     @return Temperature - Degrees C (lsb = 0.0625)
*/
int16 TempRead (uint8 Address)
{
	uint8 Dummy;
	uint16 ReadData;
	int16 ReturnValue;

	ReturnValue = 0;
	ReadData = 0;

	SDO_TRIS = 1;						//Disable output pin

	//Set the CS for the selected sensor
	switch(Address)
	{
		case 0:
			RIGHT_FRONT_CS = 0;
			break;
			
		case 1:
			LEFT_FRONT_CS = 0;
			break;
			
		case 2:
			RIGHT_REAR_CS = 0;
			break;
			
		case 3:
			LEFT_REAR_CS = 0;
			break;
			
		default:
			//Should never get to this condition
			RIGHT_FRONT_CS = 1;
			LEFT_FRONT_CS = 1;
			RIGHT_REAR_CS = 1;
			LEFT_REAR_CS = 1;
	}	

	Delay10KTCYx(10);					//Delay about 12 msec for the CS to set

	Dummy = SSPBUF;						//Dummy read to clear the BF bit

	SSPBUF = 0;							//Dummy write to start the data transfer

	while(SSPSTATbits.BF == 0);			//Wait until the data has been read

	ReadData = (uint16)SSPBUF;			//Read the first byte
	ReturnValue = (ReadData << 8);

	SSPBUF = 0;							//Dummy write to start the data transfer

	while(SSPSTATbits.BF == 0);			//Wait until the data has been read

	ReadData = (uint16)SSPBUF;			//Read the second byte
	ReturnValue += (uint16)ReadData;

	//Deselect all sensors
	RIGHT_FRONT_CS = 1;
	LEFT_FRONT_CS = 1;
	RIGHT_REAR_CS = 1;
	LEFT_REAR_CS = 1;

	SDO_TRIS = 1;						//Disable output pin
	
	return (ReturnValue/8);				//Align data and return.  Divide by 8 instead of shift 3 to the 
										//right to preserve the sign.
}




/**
	Read the vibration sensor at startup and use this value as the calibration value.  This
	is necessary to remove the earth's gravity acceleration from the data

  	@return void (calibration values are saved to global variables)

*/
void VibSensorCal(void)
{
		//Read the Right front vibration data to determine calibration
		ADCON0 = 0b00000001;		//Select the channel
		ADCON0bits.GO_DONE = 1;		//Start the conversion

		while(ADCON0bits.GO_DONE == 1);		//Wait until the conversion is complete

		RightFrontCal = ((uint16)ADRESH << 8) + ((uint16)ADRESL);
	

		//Read the Left front vibration data to determine calibration
		ADCON0 = 0b000000101;		//Select the channel
		ADCON0bits.GO_DONE = 1;		//Start the conversion

		while(ADCON0bits.GO_DONE == 1);		//Wait until the conversion is complete

		LeftFrontCal = ((uint16)ADRESH << 8) + ((uint16)ADRESL);


		//Read the Right rear vibration data to determine calibration
		ADCON0 = 0b00001001;		//Select the channel
		ADCON0bits.GO_DONE = 1;		//Start the conversion

		while(ADCON0bits.GO_DONE == 1);		//Wait until the conversion is complete

		RightRearCal = ((uint16)ADRESH << 8) + ((uint16)ADRESL);


		//Read the Left rear vibration data to determine calibration
		ADCON0 = 0b00001101;		//Select the channel
		ADCON0bits.GO_DONE = 1;		//Start the conversion

		while(ADCON0bits.GO_DONE == 1);		//Wait until the conversion is complete

		LeftRearCal = ((uint16)ADRESH << 8) + ((uint16)ADRESL);
}



