;***************************************;***************************************
;                                                                   
;    Filename:        					Servo Position Controller.asm 
;    Date:         						8/8/2013                 
;    File Version:       				V0.1                        
;                                                              
;    Author:                   			jeff bachiochi            
;    Company:							imagine that                    
;                            			www.imaginethatnow.com              
;                                                                        
;***************************************;***************************************
;                                                                        
;    Files Required: P12F1822.INC                                         
;                                                                          
;***************************************;***************************************
;                                                                          
;    Notes: 	This project makes use of any DC motor and turns it into a
;		180 degree RC servo motor taking an RC pulse input.  Two (or three)
;		external switches (opto, hall, etc.) are used to define end points
;		(and center). A shaft encoder is not necessary as position is based on 
;		time.
;                                                                         
;***************************************;***************************************
;                                                                           
;    Revision History:                                                       
;                                                                             
;***************************************;***************************************
;
	list		p=12F1822      			; list directive to define processor
	#include	<p12F1822.inc> 			; processor specific variable definitions
;
;***************************************;***************************************
;
; CONFIGURATION WORD SETUP
;
; The 'CONFIG' directive is used to embed the configuration word within the 
; .asm file. The lables following the directive are located in the respective 
; .inc file.  See the data sheet for additional information on configuration 
; word settings.
;
;***************************************;***************************************   
;
    __CONFIG _CONFIG1, _FOSC_INTOSC & _WDTE_OFF & _PWRTE_OFF & _MCLRE_OFF & _CP_OFF & _CPD_OFF & _BOREN_OFF & _CLKOUTEN_OFF & _IESO_OFF & _FCMEN_OFF
    __CONFIG _CONFIG2, _WRT_OFF & _PLLEN_ON & _STVREN_OFF & _BORV_19 & _LVP_ON
;
;***************************************;***************************************
;
; 	CONSTANTS
;
;***************************************;***************************************
;
;;;;;;	PORTA
FWD			EQU		0					; forward output to H-bridge (CW)
MAX			EQU		1					; maximum (CW) end point input
BKD			EQU		2					; backward output to H-bridge (CCW)
SERVO		EQU		3					; RC pulse input
CENTER 		EQU		4					; center point input
MIN			EQU		5					; minimum (CCW) end point input
;
;;;;;	Flags
ServoInput	EQU		0					; 0=no input, 1=RC input
OnTarget	EQU		1					; 0=need to move, 1=stay here
Positive	EQU		2					; 0=move negative, 1=move positive
Min_Sw		EQU		3					; 0=open, 1=closed
Center_Sw	EQU		4					; 0=open, 1=closed
Max_Sw		EQU		5					; 0=open, 1=closed
;
MinDegree	EQU		D'0'
MaxDegree	EQU		D'180'
DegreeSpan	EQU		MaxDegree-MinDegree
MinOnTime	EQU		D'500' 			; uS
MaxOnTime	EQU		D'1500'				; uS
TimeSpan	EQU		MaxOnTime-MinOnTime
Reload_PWMPeriod	EQU		0xFF		; full speed
Hysteresis	EQU		2					; used with delta
;
;***************************************;***************************************
;
; VARIABLE DEFINITIONS
;
; Available Data Memory divided into Bank 0-15.  Each Bank may contain 
; Special Function Registers, General Purpose Registers, and Access RAM 
;
;***************************************;***************************************
;
    CBLOCK 0x20 						; bank0 GPR variable register locations
	AARGB0								; varaiables for math routines	
	AARGB1								;
	AARGB2								;	
	AARGB3								;
	BARGB0								;
	BARGB1								;
	REMB0								;
	REMB1								;
	TEMPB0								;
	TEMPB1								;
	LOOPCOUNT							;
    ENDC
;
;		common RAM @ 0x70
;
Flags       EQU    	0x70				; System flags        	
OnTimeH		EQU		0x71				; RC pulse width measurement in uS
OnTimeL		EQU		0x72				; RC pulse width measurement in uS
Req_Position	EQU	0x73				; requested position based on RC pulse (0-180)
Act_Position	EQU	0x74				; present position (0-180)
PWMPeriod	EQU		0x75				; PWM ON-Time (motor drive)
PositiveTime	EQU	0x76				; time for 180 degree motion (CW)
NegativeTime	EQU	0x77				; time for 180 degree motion (CCW)
TempH		EQU		0x78				; temporary register 				
TempL		EQU		0x79				; temporary register
Delta		EQU		0x7A				; difference in degrees Req_Position:Act_Position
Tick		EQU		0x7C				; 1024 uS interrupts
;
;***************************************;***************************************
;
; RESET VECTOR
;
;***************************************;***************************************
;
    	ORG		0x0000         			; processor reset vector
    	nop			           			; When using debug header, first inst.
    				               		; may be passed over by ICD2.
    	goto    Init               		; branch to initialization  
;
;***************************************;***************************************
;
; INTERRUPT SERVICE ROUTINE
;
;	Change Of State
;		Servo used to determine rise and fall of the RC pulse width
;		Min used to update Flag.Min_Sw with min input state
;		Center used to update Flag.Center_Sw with center input state
;		Max used to update Flag.Max_Sw with max input state
;
;	TMR1
;		used to measure RC pulse width
;
;	TMR0
;		used to measure the motors movements
;
;***************************************;***************************************
;
    	ORG      0x0004
INT    	
;    	
INT_Chk4COS
		btfsc	INTCON,	IOCIF			; skip next if NO COS
		call	INT_COS					; else go see which one(s)
;
INT_Chk4TMR1
		banksel	PIR1
		btfsc	PIR1,	TMR1IF			; skip next if NO overflow
		call	INT_TMR1				; else save pulse width time
;
INT_Chk4TMR0
		btfss	INTCON,	TMR0IE			; skip next if Timer0 is enabled
		goto	INT_Exit				; else its not being used so go on	
;
		btfsc	INTCON,	TMR0IF			; skip next if NO overflow			
		call	INT_TMR0				; else keep track of the overflows
;
INT_Exit			
    	RETFIE                  		; return from interrupt
;    
;***************************************;***************************************
;
; 	Change Of State Interrupts			
;
; 	PreCondition:
;		IOCAF.SERVO
;		IOCAF.CENTER
;		IOCAF.MAX
;		IOCAF.MIN
;		PORTA.SERVO
;		PORTA.CENTER
;		PORTA.MAX
;		PORTA.MIN
;
;	Changes:
;		TMR1H:L
;		Flags.ServoInput
;		Flags.Center_Sw
;		Flags.Max_Sw
;		Flags.Min_Sw
;		T1CON.TMR1ON
;		OnTimeH:L
;		Act_Position
;		INTCON.IOCIF
;
; 	Calls:          
;		BrakeMotor
;
;***************************************;***************************************
; 
INT_COS
;
;***************************************
;
INT_COS_Chk4Servo
		banksel	IOCAF
		btfss	IOCAF,	SERVO			; skip next if SERVO interrupt
		goto	INT_COS_Chk4Servo_Exit
;
INT_COS_Servo
		banksel	PORTA
		btfss	PORTA,	SERVO			; skip next if SERVO is high
		goto	INT_COS_Servo_L	
;
INT_COS_Servo_H
		banksel	TMR1L
		clrf	TMR1L					; clear the timer and let it run
		clrf	TMR1H
		bsf		Flags,	ServoInput		; flag ServoInput as running
;		
		banksel	IOCAF
		bcf		IOCAF,	SERVO			; clear this interrupt
		goto	INT_COS_Chk4Servo_Exit
;
INT_COS_Servo_L	
		banksel	TMR1L
		bcf		T1CON,	TMR1ON			; stop timer1
		movf	TMR1L,	W				; save the RC pulse high time
		movwf	OnTimeL
		movf	TMR1H,	W
		movwf	OnTimeH	
		bsf		T1CON,	TMR1ON			; start timer1	
		bsf		Flags,	ServoInput		; flag ServoInput as running
;		
		banksel	IOCAF					
		bcf		IOCAF,	SERVO			; clear this interrupt	
;
INT_COS_Chk4Servo_Exit
;
;***************************************
;
INT_COS_Chk4Center
		btfss	IOCAF,	CENTER			; skip next if CENTER interrupt
		goto	INT_COS_Chk4Center_Exit
;
INT_COS_Chk4Rising
		banksel	PORTA
		btfss	PORTA,	CENTER			; skip next if now high (rising edge)
		goto	INT_COS_CenterFalling	; else now low (falling edge)	
;
INT_COS_CenterRising
		bcf		Flags,	Center_Sw		; Center_Sw is open
		banksel	IOCAF
		bcf		IOCAF,	CENTER			; clear this interrupt
		goto	INT_COS_Chk4Center_Exit		
;
INT_COS_CenterFalling
		movlw	D'90'		
		movwf	Act_Position			; we must be at the 90 degree position
		bsf		Flags,	Center_Sw		; Center_Sw is closed
		banksel	IOCAF
		bcf		IOCAF,	CENTER			; clear this interrupt	
;
INT_COS_Chk4Center_Exit
;
;***************************************
;
INT_COS_Chk4Min	
		btfss	IOCAF,	MIN				; skip next if MIN interrupt
		goto	INT_COS_Chk4Min_Exit
;
INT_COS_Chk4MinRising
		banksel	PORTA
		btfss	PORTA,	MIN				; skip next if now high (rising edge)
		goto	INT_COS_MinFalling		; else now low (falling edge)	
;
INT_COS_MinRising
		bcf		Flags,	Min_Sw			; Min_Sw is open
		banksel	IOCAF
		bcf		IOCAF,	MIN				; clear this interrupt
		goto	INT_COS_Chk4Min_Exit
;
INT_COS_MinFalling
		call	BrakeMotor
;		
		movlw	0
		movwf	Act_Position			; we must be at the 0 degree position
		bsf		Flags,	Min_Sw			; Min_Sw is open	
		banksel	IOCAF		
		bcf		IOCAF,	MIN				; clear this interrupt	
;
INT_COS_Chk4Min_Exit
;
;***************************************
;	
;
INT_COS_Chk4Max	
		btfss	IOCAF,	MAX				; skip next if MIN interrupt
		goto	INT_COS_Chk4Max_Exit
;
INT_COS_Chk4MaxRising
		banksel	PORTA
		btfss	PORTA,	MAX				; skip next if now high (rising edge)
		goto	INT_COS_MaxFalling		; else now low (falling edge)	
;
INT_COS_MaxRising
		bcf		Flags,	Max_Sw			; Max_Sw is open
		banksel	IOCAF
		bcf		IOCAF,	MAX				; clear this interrupt
		goto	INT_COS_Chk4Max_Exit
;
INT_COS_MaxFalling
		call	BrakeMotor
;
		movlw	D'180'
		movwf	Act_Position			; we must be at the 180 degree position		
		bsf		Flags,	Max_Sw			; Max_Sw is open
		banksel	IOCAF		
		bcf		IOCAF,	MAX				; clear this interrupt	
;
INT_COS_Chk4Max_Exit
;
;***************************************
;	
INT_COS_Exit
		bcf		INTCON,	IOCIF			; clear this interrupt			
		return 
;    
;***************************************;***************************************
;
; 	Timer 1 overflow Interrupts			; Fosc/8/65535 (65mS)
;
;	If we get here there has been no RC pulse input so flag this fact
;
; 	PreCondition:
;		none
;
;	Changes:
;		Flags.ServoInput
;		PIR1.TMR1IF
;
; 	Calls:          
;		none
;
;***************************************;***************************************
; 
INT_TMR1
		bcf		Flags,	ServoInput		; flag ServoInput as NOT running
;		banksel	PIR1					; not needed, set in INT
		bcf		PIR1,	TMR1IF			; clear this interrupt		
		return
;    
;***************************************;***************************************
;
; 	Timer 0 overflow Interrupt			; Fosc/32/256 (1024 uS)
;
;	This interrupt used to measure time of motor movement for 
;		CW rotation (0-180 degrees) and CCW rotation (180-0 degrees)
;	and
;		the time to move 1 degree based on rotation time / 180 
;
; 	PreCondition:
;		Tick
;		Flags.Positive
;		PositiveTime
;		NegativeTime
;
;	Changes:
;		Act_Position
;		Tick
;		TempH:L
;		INTCON.TMR0IF
;
; 	Calls:          
;		none
;
;***************************************;***************************************
; 
INT_TMR0
		btfss	Flags,	Positive		; skipnext if the rotation is positive
		goto	INT_TMR0_Negative		; else its negative
;
INT_TMR0_Positive						; CW
		movf	Tick,	W
		subwf	PositiveTime,	W		; WREG=PositiveTime-Tick
		btfss	STATUS,	Z				; skip next if PositiveTime=Tick
		goto	INT_TMR0_IncTick		; else we haven't reach 1 degree yet
;
		clrf	Tick					; clear the counter
		incf	Act_Position,	F		; adjust Act_Position as we go CW
		goto	INT_TMR0_Continue		; go on
;
INT_TMR0_Negative						; CCW
		movf	Tick,	W
		subwf	NegativeTime,	W		; WREG=PositiveTime-Tick
		btfss	STATUS,	Z				; skip next if PositiveTime=Tick
		goto	INT_TMR0_IncTick		; else we haven't reach 1 degree yet
;
		clrf	Tick					; clear the counter
		decf	Act_Position,	F		; adjust Act_Position as we go CCW
		goto	INT_TMR0_Continue		; go on
;
INT_TMR0_IncTick
		incf	Tick,	F				; next count
;
INT_TMR0_Continue		
		incfsz	TempL,	F				; skip next if next count (180 degree movement) <> rollover
		goto	INT_TMR0_Exit
;
		incf	TempH,	F				; next count (180 degree movement)		
;
INT_TMR0_Exit			
		bcf		INTCON,	TMR0IF			; clear this interrupt
		return				  
;    
;***************************************;***************************************
;
; 	Initialization
;
;***************************************;***************************************
;
Init
;
Init_OSCCON
		banksel	OSCCON
		movlw	b'11110000'				; PLL enabled 32MHz
		movwf	OSCCON
;
Init_OPTION_REG
		banksel	OPTION_REG
		movlw	b'01000100'				; weak pullups enabled, TMR0 pre=/32
		movwf	OPTION_REG
;
Init_PORTA
		banksel	LATA
		movlw	b'11111010'				; x, x, MAX=1, CENTER=1, SERVO=1, BKD=0, MIN=1, FWD=0
		movwf	LATA
;	
		banksel	TRISA
		movlw	b'11111010'				; x, x, MAX=1, CENTER=1, SERVO=1, BKD=0, MIN=1, FWD=0
		movwf	TRISA
	
		banksel	ANSELA
		movlw	b'11101000'				; x, x, x, CENTER=0, x, BKD=0, MIN=0, FWD=0
		movwf	ANSELA		
;
Init_PWM
		banksel	CCP1CON	
		movlw	b'00001100'				; P1M<1:0>=00, DC1B<1:0>=00, CCP1M<3:0>=1100
		movwf	CCP1CON
;
		banksel	PSTR1CON
		movlw	b'00000001'				; x, x, x, STR1SYNC=0, STR1D=0, STR1C=0, STR1B=0, STR1A=1,
		movwf	PSTR1CON
;
		banksel	T2CON
		movlw	b'00000100'				; x, T2OUTPS<3:0>=000, TMR2ON=1, T2CKPS<1:0>=00 (31.250Hz)
		movwf	T2CON
;
		banksel	PR2
		movlw	b'11111111'				; 10-bits
		movwf	PR2
;
		banksel	CCPR1L
		movlw	Reload_PWMPeriod		; set PWM (using full speed)
		movwf	PWMPeriod				 
		movwf	CCPR1L
;
Init_Timer0
										; nothing to do here	
;
Init_Timer1
		banksel	T1CON
		movlw	b'00110001'				; TMR1CS<1:0>=00, T1CKPS<1:0>=11, T1OSCEN=0, T1SYNC=0, x, TMR1ON=1
		movwf	T1CON
;
		banksel	PIR1
		bcf		PIR1,	TMR1IF						
;
Init_IOC
		banksel	IOCAP					; rising interrupt
		movlw	b'00111010'				; x, x, IOCAP5:0=001000
		movwf	IOCAP
;
		banksel	IOCAN					; falling interrupt
		movlw	b'00111010'				; x, x, IOCAP5:0=111010
		movwf	IOCAN
;
		banksel	IOCAF
		clrf	IOCAF					; clear COS bit flags
		bcf		INTCON,	IOCIF			; clear COS flag
;
		clrf	Flags
;		
Init_INTERRUPTS
		banksel	PIE1
		bsf		PIE1,	TMR1IE			; allow timer1 interrupts
		bsf		INTCON,	IOCIE			; allow COS interrupts
		bsf		INTCON,	PEIE			; enable interrupts
		bsf		INTCON,	GIE				; enable interrupts	
;	
Init_Motor
		call	GotoMin					; let's start at 0 degrees (Min_Sw=1)
;		
		call	TimeToMax				; time the move to 180 degrees (Max_Sw=1)
;
		call	TimeToMin				; time the move to 0 degrees (Min_Sw=1)
;
		movlw	D'90'					; this will set up for 'GoToPosition'
		movwf	Delta					; let's move to 90 degrees
		movwf	Req_Position			; it's where we want to start
		bcf		Flags,	OnTarget		; we will need to move
		bsf		Flags,	Positive		; we will want to move Positive (CW)
		call	GoToPosition			; move delta degrees
;						
Init_Exit
		goto	Main   					; go servo based on RC pulse width
;    
;***************************************;***************************************
;
; 	MAIN PROGRAM
;
;***************************************;***************************************
;
Main
		call	Servo					; just one thing to do 
;		
    	goto	Main
;    
;***************************************;***************************************
;
; 	Servo								- move DC motor based on RC pulse width
;
; 	PreCondition:
;		Flags.ServoInput
;		Flags.OnTarget
;		Flags.Positive
;		Flags.Max_Sw
;		Flags.Negative
;		Flags.Min_Sw
;
;	Changes:
;		none
;
; 	Calls:          
;		IdleMotor						- set motor to safe state
;		BrakeMotor						- set motor to stop
;		CalcRequestedPosition			- convert OnTimeH:L to Degrees
;		CalcDelta						- find the difference between
;											Req_Position and Act_Position
;		GoToPosition					- move delta degrees
;		EnableMotorPositive				- set motor to move Positive (CW)
;		EnableMotorNegative				- set motor to move Negative (CCW)
;
;***************************************;***************************************
;
Servo 
;
Servo_Chk4RC
		banksel	CCPR1L
		btfsc	Flags,	ServoInput		; skip next if ServoInput=0 (no RC) 
		goto	Servo_Chk4OnTarget
;
Servo_NoRC								; no output 
		call	IdleMotor				; set motor to safe state
;		
		goto	Servo_Exit				; done here
;
Servo_Chk4OnTarget
		call	CalcRequestedPosition	; convert OnTimeH:L to Degrees
;
		call	CalcDelta				; find the difference between
										; Req_Position and Act_Position
;		
		call	GoToPosition			; move delta degrees
;
		banksel	CCPR1L 
		btfss	Flags,	OnTarget		; skip next if delta=0
		goto	Servo_Chk4Direction		; go on		
;
Servo_OnTarget							; stop
		call	BrakeMotor 				; set motor to stop
;		
		goto	Servo_Exit				; done here
;
Servo_Chk4Direction
		btfss	Flags,	Positive		; skip next if we need to move positive
		goto	Servo_Negative			; go on		
;
Servo_Positive							; positive
		btfsc	Flags,	Max_Sw			; skip next if Max_Sw is open
		goto	Servo_Exit				; done here
;		
		call	EnableMotorPositive		; set motor to move Positive (CW)
;						
		goto	Servo_Exit				; done here
;
Servo_Negative							; negative
		btfsc	Flags,	Min_Sw			; skip next if Min_Sw is open
		goto	Servo_Exit
;
		call	EnableMotorNegative		; set motor to move Negative (CCW)
;		
		goto	Servo_Exit				; done here
;
Servo_Exit
		return
;
;***************************************;***************************************
;
;	Idle Motor							- set motor to safe state	
;
; 	PreCondition:
;		none
;
;	Changes:
;		CCPR1L
;		CCP1CON
;		LATA.FWD
;
; 	Calls:          
;		none
;
;***************************************;***************************************
;		
IdleMotor
		banksel	CCPR1L
		clrf	CCPR1L 
		movlw	0x0C
		movwf	CCP1CON
		banksel	LATA		
		bcf		LATA,	FWD
		return		
;
;***************************************;***************************************
;
;	Brake Motor							- set motor to stop
;
; 	PreCondition:
;		none
;
;	Changes:
;		CCPR1L
;		CCP1CON
;		LATA.FWD
;
; 	Calls:          
;		none
;
;***************************************;***************************************
;		
BrakeMotor	
		banksel	CCPR1L	
		clrf	CCPR1L
		movlw	0x0E
		movwf	CCP1CON
		banksel	LATA		
		bsf		LATA,	FWD
		return		
;
;***************************************;***************************************
;
;	Enable Motor Positive				- set motor to move Positive (CW)
;
; 	PreCondition:
;		PWMPeriod
;
;	Changes:
;		CCPR1L
;		CCP1CON
;		LATA.FWD
;
; 	Calls:          
;		none
;
;***************************************;***************************************
;
EnableMotorPositive	
		banksel	CCPR1L
		movf	PWMPeriod,	W
		movwf	CCPR1L
		movlw	0x0E
		movwf	CCP1CON	
		banksel	LATA		
		bsf		LATA,	FWD	
		return
;
;***************************************;***************************************
;
;	Enable Motor Negative				- set motor to move Negative (CCW)
;
; 	PreCondition:
;		PWMPeriod
;
;	Changes:
;		CCPR1L
;		CCP1CON
;		LATA.FWD
;
; 	Calls:          
;		none
;
;***************************************;***************************************
;		
EnableMotorNegative
		banksel	CCPR1L		
		movf	PWMPeriod,	W
		movwf	CCPR1L
		movlw	0x0C
		movwf	CCP1CON	
		banksel	LATA
		bcf		LATA,	FWD	
		return			
;
;***************************************;***************************************
;
;	Calculate the Requested Position (time to degrees)
;
; 	PreCondition:
;		MinOnTimeH:L
;		OnTimeH:L
;		MinDegree
;		MaxOnTimeH:L
;		MaxDegree
;
;	Changes:
;		Req_Position
;		AARGB0:4
;		BARGB0:1
;
; 	Calls:          
;		SUB1616							- word-word
;		FXM1616U						- word*word
;		FXD1616U						- word/word
;
;***************************************;***************************************
;
CalcRequestedPosition
;
CalcRequestedPosition_Chk4LTMin
		banksel	AARGB0
		movlw	Low MinOnTime			; minimum RC pulse width
		movwf	BARGB1
		movlw	High MinOnTime
		movwf	BARGB0		
		movf	OnTimeL,	W			; present pulse width
		movwf	AARGB1
		movf	OnTimeH,	W
		movwf	AARGB0
		call	SUB1616					; word-word
;
		btfss	AARGB0,	7				; skip next if borrow (OnTime<MinOnTime)
		goto	CalcRequestedPosition_Chk4Min	; go on
;
CalcRequestedPositionMin				; make Position=MinDegree
		movlw	MinDegree
		movwf	Req_Position			; make it so
		goto	CalcRequestedPosition_Exit	; done here
;
CalcRequestedPosition_Chk4Min
		movf	AARGB0,	W
		iorwf	AARGB1,	W
		btfsc	STATUS,	Z				; skip next if <> (OnTime<>MinOnTime)
		goto	CalcRequestedPositionMin	; go on		
;
CalcRequestedPosition_Chk4GTMax		
		movf	OnTimeL,	W			; present pulse width
		movwf	BARGB1
		movf	OnTimeH,	W
		movwf	BARGB0
		movlw	Low MaxOnTime			; maximum RC pulse width
		movwf	AARGB1
		movlw	High MaxOnTime
		movwf	AARGB0				
		call	SUB1616					; word-word		
;		
		btfss	AARGB0,	7				; skip next if borrow (MaxOnTime<OnTime)
		goto	CalcRequestedPosition_Chk4Max	; go on
;
CalcRequestedPositionMax				; make Position=MaxDegree
		movlw	MaxDegree
		movwf	Req_Position			; make it so
		goto	CalcRequestedPosition_Exit	; done here		
;
CalcRequestedPosition_Chk4Max
		movf	AARGB0,	W
		iorwf	AARGB1,	W		
		btfsc	STATUS,	Z				; skip next if <> (MaxOnTime<>OnTime)
		goto	CalcRequestedPositionMax	; go on
;
CalcRequestedPosition_Calc
		movlw	Low MinOnTime			; minimum RC pulse width
		movwf	BARGB1
		movlw	High MinOnTime
		movwf	BARGB0	
		movf	OnTimeL,	W			; present pulse width
		movwf	AARGB1
		movf	OnTimeH,	W
		movwf	AARGB0
		call	SUB1616					; word-word (pulse width in uS)	
	
		movlw	D'18'
		movwf	BARGB1	
		clrf	BARGB0				
		call	FXM1616U				; word*word ARG (uS scaled up by 18 degrees) = difference * 18		
;
		movlw	Low (TimeSpan/D'10')	; maximum RC pulse width -
										; minimum RC pulse width / 10
		movwf	BARGB1		
		movlw	High (TimeSpan/D'10')
		movwf	BARGB0
		movf	AARGB3,	W				; ARG
		movwf	AARGB1
		movf	AARGB2,	W
		movwf	AARGB0
		call	FXD1616U				; word/word ARG (degrees) = ARG / (TimeSpan/10)				
;
		movf	AARGB1,	W			
		movwf	Req_Position			; save ARG as Req_Position
;
CalcRequestedPosition_Exit				
		return
;
;***************************************;***************************************
;
;	Calculate the Delta (requested-actual)
;
; 	PreCondition:
;		Act_Position
;		Req_Position
;		Hysteresis
;
;	Changes:
;		Delta
;		Flags.Positive
;		Flags.OnTarget
;
; 	Calls:          
;		none
;
;***************************************;***************************************
;
CalcDelta
		movf	Act_Position,	W
		subwf	Req_Position,	W		; WREG=Delta
		btfss	STATUS,	C				; skip next if Act_Position<=Req_Position
		goto	CalcDelta_Negative		; go on
;
CalcDelta_Positive
		bsf		Flags,	Positive		; move might be Positive
		movwf	Delta					; difference
		sublw	Hysteresis				; WREG=Hyteresis-Delta
		btfsc	STATUS,	C				; skip next if Delta>Hysteresis
		goto	CalcDelta_NoMove		; go on
;		
		bcf		Flags,	OnTarget		; flag OnTarget as NOT
		bsf		Flags,	Positive		; flag direction as Positive
		goto	CalcDelta_Exit			; done here		

CalcDelta_Negative
		bcf		Flags,	Positive		; move might be Negative
		comf	WREG,	W
		movwf	Delta		
		sublw	Hysteresis
		btfsc	STATUS,	C				; skip next if Delta>Hysteresis
		goto	CalcDelta_NoMove		; go on
;				
		bcf		Flags,	OnTarget		; flag OnTarget as NOT
		bcf		Flags,	Positive		; flag direction as Negative		
		goto	CalcDelta_Exit			; done here	

CalcDelta_NoMove
		bsf		Flags,	OnTarget		; no need to move
		goto	CalcDelta_Exit			; done here			
;
CalcDelta_Exit		
		return
;
;***************************************;***************************************
;
;		GotoMin
;
; 	PreCondition:
;		PORTA.MIN
;		Flags.Min_Sw
;
;	Changes:
;		none
;
; 	Calls:          
;		EnableMotorNegative
;		BrakeMotor	
;
;***************************************;***************************************
;		
GotoMin
		banksel	PORTA
		btfss	PORTA,	MIN			; skip next if Min_Sw is open
		goto	GotoMin_Exit		; done here
		
		call	EnableMotorNegative ; set motor to move Negative (CCW)
;
GotoMin_Loop
		btfss	Flags,	Min_Sw		; skip next if Min_Sw is closed
		goto	GotoMin_Loop		; go back
;
		call	BrakeMotor			; set motor to stop
;
GotoMin_Exit					
		return
;
;***************************************;***************************************
;
;		TimeToMax
;
; 	PreCondition:
;		Flags.Max_Sw
;
;	Changes:
;		INTCON.TMR0IE
;		PositiveTime
;		AARGB0:1
;		BARGB0:1
;
; 	Calls:          
;		InitTimer0 						- initialize Timer0
;		EnableMotorPositive				- set motor to move Positive (CW)
;		BrakeMotor						- set motor to stop
;		FXD1616U						- word/word
;
;***************************************;***************************************
;		
TimeToMax
		call	InitTimer0 				; initialize Timer0	
;                
		call	EnableMotorPositive		; set motor to move Positive (CW)
;
TimeToMax_Loop
		btfss	Flags,	Max_Sw			; skip next if Max_Sw is closed
		goto	TimeToMax_Loop
;
		call	BrakeMotor				; set motor to stop
;
		bcf		INTCON,	TMR0IE 			; stop Timer0
		banksel	AARGB0		
		movf	TempH,	W				; time it took to move				
		movwf	AARGB0
		movf	TempL,	W
		movwf	AARGB1
		clrf	BARGB0					; 180 degrees
		movlw	D'180'
		movwf	BARGB1
		call	FXD1616U				; AARG (time/degree) = time / degrees
;		
		movf	AARGB1,	W						
		movwf	PositiveTime			; save the time
		incf	PositiveTime,	F		; round up
		return
;
;***************************************;***************************************
;
;		TimeToMin
;
; 	PreCondition:
;		Flags.Min_Sw
;
;	Changes:
;		INTCON.TMR0IE
;		NegativeTime
;		AARGB0:1
;		BARGB0:1
;
; 	Calls:          
;		InitTimer0 						- initialize Timer0
;		EnableMotorNegative				- set motor to move Negative (CCW)
;		BrakeMotor						- set motor to stop
;		FXD1616U						- word/word
;
;***************************************;***************************************
;
TimeToMin
		call	InitTimer0 
;		                                      
		call	EnableMotorNegative		; enable negative move
;
TimeToMin_Loop
		btfss	Flags,	Min_Sw			; skip next if Min_Sw is closed
		goto	TimeToMin_Loop
;
		call	BrakeMotor
;
		bcf		INTCON,	TMR0IE 			; stop Timer0
		banksel	AARGB0
		movf	TempH,	W
		movwf	AARGB0
		movf	TempL,	W
		movwf	AARGB1
		clrf	BARGB0
		movlw	D'180'
		movwf	BARGB1
		call	FXD1616U
;		
		movf	AARGB1,	W						
		movwf	NegativeTime
		incf	NegativeTime,	F		; round up				
		return
;
;***************************************;***************************************
;
;		InitTimer0
;
; 	PreCondition:
;		none
;
;	Changes:
;		TMR0
;		TempH:L
;		Tick
;		INTCON.TMR0IF
;		INTCON.TMR0IE
;
; 	Calls:          
;		none	
;
;***************************************;***************************************
;
InitTimer0		
		banksel	TMR0					; initialize and enable Timer0
		clrf	TMR0
		clrf	TempH
		clrf	TempL
		clrf	Tick
		bcf		INTCON,	TMR0IF
		bsf		INTCON,	TMR0IE
		return		
;
;***************************************;***************************************
;
;		GoToPosition
;
; 	PreCondition:
;		Flags.OnTarget
;		Flags.Positive
;		Act_Position
;		Req_Position
;
;	Changes:
;		INTCON.TMR0IE 
;
; 	Calls:          
;		InitTimer0  					- initialize Timer0	
;		EnableMotorPositive				- set motor to move Positive (CW)
;		BrakeMotor						- set motor to stop
;		EnableMotorNegative				- set motor to move Negative (CCW)
;
;***************************************;***************************************
;
GoToPosition
		banksel	AARGB0
		btfsc	Flags,	OnTarget		; skip next if need to move
		goto	GoToPosition_Exit		; done here
;		
		btfss	Flags,	Positive		; skip next if Positive Move
		goto	NegativeMove
;
PositiveMove
		call	InitTimer0 				; initialize Timer0	
;
		call	EnableMotorPositive		; set motor to move Positive (CW)
;
PositiveMove_Loop
		movf	Act_Position,	W
		subwf	Req_Position,	W		; WREG=Req_Position-Act_Position
		btfsc	STATUS,	Z				; skip next if Act_Position<>Reg_Position
		goto	PositiveMove_Stop		; go on
;
		btfsc	STATUS,	C				; skip next if Act_Position>Req_Position
		goto	PositiveMove_Loop		; go back			
;
PositiveMove_Stop		
		call	BrakeMotor				; set motor to stop
;
		bcf		INTCON,	TMR0IE 			; stop Timer0
		goto	GoToPosition_Exit		; done here
;
NegativeMove
		call	InitTimer0 				; initialize Timer0	
;
		call	EnableMotorNegative		; set motor to move Negative (CCW)
;
NegativeMove_Loop
		movf	Req_Position,	W
		subwf	Act_Position,	W		; WREG=Act_Position-Req_Position
		btfsc	STATUS,	Z				; skip next if Req_Position<>Act_Position
		goto	NegativeMove_Stop		; go on
;
		btfsc	STATUS,	C				; skip next if Req_Position>Act_Position
		goto	NegativeMove_Loop		; go back								
;
NegativeMove_Stop		
		call	BrakeMotor				; set motor to stop
;
		bcf		INTCON,	TMR0IE 			; stop Timer0
		goto	GoToPosition_Exit		; done here
;
GoToPosition_Exit						
		return
;
;***************************************;***************************************
;
;   16/16 Bit Unsigned Fixed Point addition 16+16 -> 16
;
;	Input: 16 bit unsigned fixed point addend in AARGB0, AARGB1
; 	16 bit unsigned fixed point addend in BARGB0, BARGB1
;
; 	Use: CALL ADD1616
;
; 	Output: 16 bit unsigned fixed point sum in AARGB0, AARGB1
;
; 	Result: AARG <-- AARG + BARG
;
;***************************************;***************************************
;		
ADD1616 
		banksel	AARGB0	
		movf	BARGB1,	W
		addwf	AARGB1,	F
		movf	BARGB0,	W
		addwfc	AARGB0,	F	
		return
;
;***************************************;***************************************
;
;   16/16 Bit Unsigned Fixed Point subtraction 16-16 -> 16
;
;	Input: 16 bit unsigned fixed point minuend in AARGB0, AARGB1
; 	16 bit unsigned fixed point subtrahend in BARGB0, BARGB1
;
; 	Use: CALL SUB1616
;
; 	Output: 16 bit unsigned fixed point difference in AARGB0, AARGB1
;
; 	Result: AARG <-- AARG - BARG
;
;***************************************;***************************************
;
SUB1616
		banksel	AARGB0
		movf	BARGB1,	W
		subwf	AARGB1,	F
		movf	BARGB0,	W
		subwfb	AARGB0,	F
		return



;**********************************************************************************************
        
;       16/16 Bit Unsigned Fixed Point Divide 16/16 -> 16.16

;       Input:  16 bit unsigned fixed point dividend in AARGB0, AARGB1
;               16 bit unsigned fixed point divisor in BARGB0, BARGB1

;       Use:    CALL    FXD1616U

;       Output: 16 bit unsigned fixed point quotient in AARGB0, AARGB1
;               16 bit unsigned fixed point remainder in REMB0, REMB1

;       Result: AARG, REM  <--  AARG / BARG
;
;***************************************;***************************************
;
FXD1616U
		banksel	AARGB0
        clrf   REMB0
        clrf   REMB1
                
UDIV1616L
        movlw  D'16'
        movwf  LOOPCOUNT

LOOPU1616
        rlf   	AARGB0,	W
        rlf   	REMB1, 	F
        rlf   	REMB0, 	F
        movf  	BARGB1,	W
        subwf  	REMB1, 	F
        movf  	BARGB0,	W
        btfss	STATUS,	C
        incfsz  BARGB0,	W
        subwf  	REMB0, 	F
        btfsc	STATUS,	C
        goto   	UOK66LL
;                
        movf  	BARGB1,	W
        addwf   REMB1, 	F
        movf  	BARGB0,	W
        btfsc	STATUS,	C
        incfsz  BARGB0,	W
        addwf  	REMB0, 	F
        bcf		STATUS,	C

UOK66LL
        rlf   	AARGB1, F
        rlf   	AARGB0, F
        decfsz  LOOPCOUNT, F
        goto   	LOOPU1616
;        
        retlw  0x00
;        
;**********************************************************************************************
;
; 	16x16 Bit Unsigned Fixed Point Multiply 16x16 -> 32
; 	Input: 16 bit unsigned fixed point multiplicand in AARGB0
; 			16 bit unsigned fixed point multiplier in BARGB0
; 	Use: CALL FXM1616U
; 	Output: 32 bit unsigned fixed point product in AARGB0
; 	Result: AARG <-- AARG x BARG
;        
;**********************************************************************************************
;
FXM1616U
		banksel	AARGB0
		clrf	AARGB2 ; clear partial product
		clrf	AARGB3
		movf	AARGB0,W
		movwf	TEMPB0
		movf	AARGB1,W
		movwf	TEMPB1

UMUL1616L
		movlw	0x08
		movwf	LOOPCOUNT
		
LOOPUM1616A
        rrf   	BARGB1, F
		btfsc	STATUS,	C
		goto	ALUM1616NAP

		decfsz 	LOOPCOUNT, F
		goto	LOOPUM1616A

		movwf	LOOPCOUNT

LOOPUM1616B
        rrf   	BARGB0, F
		btfsc	STATUS,	C
		goto	BLUM1616NAP

		decfsz 	LOOPCOUNT, F
		goto	LOOPUM1616B

		clrf	AARGB0
		clrf	AARGB1
		retlw 	0x00

BLUM1616NAP
		bcf		STATUS,	C	 	
		goto	BLUM1616NA

ALUM1616NAP
		bcf		STATUS,	C
		goto	ALUM1616NA

ALOOPUM1616
        rrf   	BARGB1, F
		btfss	STATUS,	C
		goto	ALUM1616NA

		movf	TEMPB1,W
		addwf	AARGB1, F
		movf	TEMPB0,W
		btfsc	STATUS,	C
		incfsz 	TEMPB0,W
		addwf	AARGB0, F

ALUM1616NA
        rrf   	AARGB0, F
        rrf   	AARGB1, F
        rrf   	AARGB2, F
		decfsz 	LOOPCOUNT, F
		goto	ALOOPUM1616

		movlw	0x08
		movwf	LOOPCOUNT

BLOOPUM1616
        rrf   	BARGB0, F
		btfss	STATUS,	C
		goto	BLUM1616NA
		
		movf	TEMPB1,W
		addwf	AARGB1, F
		movf	TEMPB0,W
		btfsc	STATUS,	C
		incfsz 	TEMPB0,W
		addwf	AARGB0, F

BLUM1616NA
        rrf   	AARGB0, F
        rrf   	AARGB1, F
        rrf   	AARGB2, F
        rrf   	AARGB3, F
		decfsz 	LOOPCOUNT, F
		goto	BLOOPUM1616

		retlw 	0x00      
;
;***************************************;***************************************
;
;	End
;
;***************************************;***************************************
;
    	END