
/*////////////////////////////////////////////////////////////////////////////
    PROJECT:    Trailer Display
    COPYRIGHT    2010 Dean J. Boman

    FILE:       Display.c
    AUTHOR:     Dean J. Boman

    OVERVIEW:
        Provides the interface to the LCD display


    REVISION HISTORY:

    Oct. 18, 2010	Import from Battery Monitor
	Oct. 19, 2010	Initial modes for trailer display
	Oct. 30, 2010	Added delay to E signal
	Jan. 8, 2013	Update for new trailer



*/////////////////////////////////////////////////////////////////////////////

// These headers must be included for required defs.
#include <string.h>
#include <math.h>
#include <delays.h>

#include "..\Include\typedef.h"
#include "..\Include\Compiler.h"
#include "..\Include\Display.h"


/**
	Write the left and right temperatures to the display.  Left is on the right three
	locations of the top line and the Right is on the right three locations on the second
	line

  	@return void

*/
void Write_temperature(uint8 LeftTemperature, uint8 RightTemperature)
{

	int8 DisplayString[3];

	sprintf(DisplayString,"%3u",LeftTemperature);

	Write_command(0b10001101);		//Point to 13th position on first line

	Write_data(DisplayString[0]);
	Write_data(DisplayString[1]);
	Write_data(DisplayString[2]);


	sprintf(DisplayString,"%3u",RightTemperature);

	Write_command(0b11001101);		//Point to 13th position on second line

	Write_data(DisplayString[0]);
	Write_data(DisplayString[1]);
	Write_data(DisplayString[2]);

}


/**
	Write the vibration data to the display in the form of a bar graph.  The first line
	is for the Left level nad the second line is for the Right level.  Each line contains an
	R or L in location one followed by 12 locations that represent a 0 to 600 vibration level.

  	@return void

*/
void Write_vibration(uint8 RightDisplayBlocks, uint8 LeftDisplayBlocks, uint8 LimitDisplayBlocks, boolean FrontFlag)
{

	uint8 DisplayIndex;

	Write_command(0b10000000);		//Point to first position on first line

	if(FrontFlag == TRUE)
	{
		Write_data('R');
		Write_data('f');
	}	
	else
	{
		Write_data('R');
		Write_data('r');
	}	

	for(DisplayIndex = 0;DisplayIndex < 11;DisplayIndex++)
	{
		if(DisplayIndex < RightDisplayBlocks)
			Write_data(0xff);
		else
		{
			if(DisplayIndex == LimitDisplayBlocks)
				Write_data(0x7c);
			else
				Write_data(0xfe);		
		}
	}

	Write_command(0b11000000);		//Point to first position on second line
	
	if(FrontFlag == TRUE)
	{
		Write_data('L');
		Write_data('f');
	}	
	else
	{
		Write_data('L');
		Write_data('r');
	}	

	for(DisplayIndex = 0;DisplayIndex < 11;DisplayIndex++)
	{
		if(DisplayIndex < LeftDisplayBlocks)
			Write_data(0xff);
		else
			if(DisplayIndex == LimitDisplayBlocks)
				Write_data(0x7c);
			else
				Write_data(0xfe);
	}
}


/**
	Writes a test message to the display

  	@return void
*/
void Write_test_message(void)
{
	Write_command(0b10000000);		//Point to first position on first line

	Write_data('T');
	Write_data('r');
	Write_data('a');
	Write_data('i');
	Write_data('l');
	Write_data('e');
	Write_data('r');
	Write_data(' ');
	Write_data('M');
	Write_data('o');
	Write_data('n');
	Write_data('i');
	Write_data('t');
	Write_data('o');
	Write_data('r');
	Write_data(' ');

	Write_command(0b11000000);		//Point to first position on second line

	Write_data('W');
	Write_data('a');
	Write_data('i');
	Write_data('t');
	Write_data('i');
	Write_data('n');
	Write_data('g');
	Write_data(' ');
	Write_data('f');
	Write_data('o');
	Write_data('r');
	Write_data(' ');
	Write_data('d');
	Write_data('a');
	Write_data('t');
	Write_data('a');
}




/**
	Initialize the HDM16216H-4 LCD per the data sheet

  	@return void

*/
void Reset_display(void)
{
	printf("Resetting the display\r");

	DelayMs(20);		//Wait for the display to wake up before sending any commands

	ConfigDataPortOut();

	WriteDataPort(0b00111000);		//Write with no busy bit check
	RS_IO = 0;
	RW_IO = 0;
	E_IO = 1;
	Delay10us(1);					//E must be true for > 500 nsec
	E_IO = 0;

	DelayMs(5);

	WriteDataPort(0b00111000);		//Write with no busy bit check
	RS_IO = 0;
	RW_IO = 0;
	E_IO = 1;
	Delay10us(1);					//E must be true for > 500 nsec
	E_IO = 0;

	DelayMs(1);   

	WriteDataPort(0b00111000);		//Write with no busy bit check
	RS_IO = 0;
	RW_IO = 0;
	E_IO = 1;
	Delay10us(1);					//E must be true for > 500 nsec
	E_IO = 0;

	Write_command(0b00111000);		//Two line mode

	Write_command(0b00001100);		//Display ON without cursor ON of blink

	Write_command(0b00000001);		//Clear display

	Write_command(0b00000110);		//Auto increment with no scroll

	printf("Done resetting the display\r");
}



/**
	Writes a command byte to the LCD.  Does not return until the command is completed.

	@param uint8 Commnad - Command word to be written to the display

  	@return void

*/
void Write_command(uint8 Command)
{
	boolean BusyBit;
	
	ConfigDataPortOut();			//Set port for output

	WriteDataPort(Command);			//Write command to port and sent control signals
	RS_IO = 0;						//Indicate that this is a command not data
	RW_IO = 0;
	E_IO = 1;
	Delay10us(1);					//E must be true for > 500 nsec
	E_IO = 0;

	ConfigDataPortIn();				//Set port for input to read busy bit

	BusyBit = TRUE;

	//Wait here until the busy bit goes low indicating the command is complete
	while(BusyBit == TRUE)
	{
		RW_IO = 1;
		E_IO = 1;
		Delay10us(1);				//E must be true for > 500 nsec

		if((ReadDataPort() & 0x80) == 0)
			BusyBit = FALSE;

		E_IO = 0;
		Delay10us(1);				//E must be true for > 500 nsec
	}
}


/**
	Writes a data byte to the LCD.  Does not return until the command is completed.

	@param uint8 Data - Data word to be written to the display

  	@return void

*/
void Write_data(uint8 Data)
{
	boolean BusyBit;
	
	ConfigDataPortOut();			//Set port for output

	WriteDataPort(Data);			//Write data to port and sent control signals
	RS_IO = 1;						//Indicate that this is data not a command
	RW_IO = 0;
	E_IO = 1;
	Delay10us(1);					//E must be true for > 500 nsec
	E_IO = 0;
	Delay10us(1);					//E must be true for > 500 nsec

	ConfigDataPortIn();				//Set port for input to read busy bit

	BusyBit = TRUE;

	//Wait here until the busy bit goes low indicating the command is complete

	while(BusyBit == TRUE)
	{
		RS_IO = 0;
		RW_IO = 1;
		E_IO = 1;
		Delay10us(1);				//E must be true for > 500 nsec

		if((ReadDataPort() & 0x80) == 0)
			BusyBit = FALSE;

		E_IO = 0;
		Delay10us(1);				//E must be true for > 500 nsec
	}
}



/**
	Configures the display data port for output


  	@return void

*/
void ConfigDataPortOut(void)
{
	//Configure the parallel port to the LCD as output
	TRISC = TRISC & 0b11000000;
	TRISB = TRISB & 0b11111100;
}


/**
	Configures the display data port for input


  	@return void

*/
void ConfigDataPortIn(void)
{
	//Configure the parallel port to the LCD as input
	TRISC = TRISC | 0b00111111;
	TRISB = TRISB | 0b00000011;
}


/**
	Writes data to the display data port.  Assumes that the port is pre-configured
	for output

	@param uint8 Data - Data word to be written to the port

  	@return void

*/
void WriteDataPort(uint8 Data)
{
	PORTC = (Data & 0b11111100) >> 2;
	PORTB = (PORTB & 0b11111100) | (Data & 0b00000011);
}


/**
	Reads data fromthe display data port.  Assumes that the port is pre-configured
	for input

  	@return uint8 - port data

*/
uint8 ReadDataPort(void)
{
	uint8 DataIn;

	DataIn = ((PORTC & 0b00111111) << 2);
	DataIn |= (PORTB & 0b00000011);
}


