/*////////////////////////////////////////////////////////////////////////////
    PROJECT:    Trailer Display
    COPYRIGHT    2010 Dean J. Boman

    FILE:       Main.c
    AUTHOR:     Dean J. Boman

    OVERVIEW:
        Implementation of the main routine for the trailer vibration detection
		system.

    REVISION HISTORY:

    Sept. 13, 2010	Initial development based on the APRS code base
    Jan. 8, 2013	Update for new trailer





*/////////////////////////////////////////////////////////////////////////////
#define BAUD_RATE       (19200)     // bps

// These headers must be included for required defs.
#include <p18cxxx.h>		//This brings in #include <p18f2620.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "..\Include\Compiler.h"
#include "..\Include\Serial.h"
#include "..\Include\typedef.h"
#include "..\Include\display.h"

#define VERSION "3.0 Jan. 12, 2013   \r\n"

#define TIMERDELTA 1000				//Timer1 tick rate is 1 usec so generate 1 msec interrupt

//Determine SPBRG value for given baud rate
#define SPBRG_VAL   ( ((INSTR_FREQ/BAUD_RATE)/16) - 1)

#if(SPBRG_VAL > 255)
    #error "Calculated SPBRG value is out of range for current CLOCK_FREQ."
#endif


//Set configuration fuses
#pragma config OSC=INTIO67, FCMEN=OFF, IESO=OFF, PWRT=ON, WDT=ON, WDTPS = 256, LVP=OFF, DEBUG=OFF, PBADEN=OFF
//Note the watchdog is set for a 1 sec time out


//Prototypes for local functions
static void InitializeBoard(void);
uint16 sysCRC16 (uint8 *buffer, uint16 length, uint16 crc);
void DataProcess(void);


//Define the states for the Serial state machine
typedef enum _SERIAL_STATE
{
	SERIAL_START,
	SERIAL_START_A,
	SERIAL_START_T,
	SERIAL_START_2A,
	SERIAL_START_M,
	SERIAL_MESSAGE,
	SERIAL_CRC,
	SERIAL_DECODE,
}SERIAL_STATE;


SERIAL_STATE SerialMode;

boolean HalfSecFlag;
boolean FIFOError;
boolean TemperatureAlarmFlag;
boolean VibrationAlarmFlag;
boolean DataValid;

uint8 SerialBuffer[50];

uint8 SerialTokenCount;
uint8 SerialMessageIndex;
uint8 SerialToken[10];
uint8 OverLimitCounter;
uint8 VibrationLimit;
uint8 RightFrontVibrationDisplay;
uint8 LeftFrontVibrationDisplay;
uint8 RightRearVibrationDisplay;
uint8 LeftRearVibrationDisplay;
uint8 RightFrontTemperatureF;
uint8 LeftFrontTemperatureF;
uint8 RightRearTemperatureF;
uint8 LeftRearTemperatureF;
uint16 TimerCompare;
uint16 HalfSecCounter;
uint16 CRC_value;
uint16 Limit_setting;
uint16 CalculatedCRC;
uint16 RightFrontVibration;
uint16 LeftFrontVibration;
uint16 RightRearVibration;
uint16 LeftRearVibration;

int16 RightFrontTemperature;
int16 LeftFrontTemperature;
int16 RightRearTemperature;
int16 LeftRearTemperature;

int16 RecoveredCRC;

//Interrupt service routine for all interrupts.  
#pragma interrupt HighISR

void HighISR(void)
{
    //Determine if interrupt was caused by a CCP1 compare.  CCP1 interrupts occur
	//every 1 msec and are used to basic time keeping.  In this application this is only used 
	//to flash the I'm alive LED for debug purposes
	if(PIR1bits.CCP1IF)
    {
		//We have had a 1 msec interrupt
		HalfSecCounter++;
		if(HalfSecCounter >= 500)
		{
			HalfSecFlag = TRUE;		
			HalfSecCounter = 0;
		}

		//Set the compare register for the next interval
		TimerCompare += TIMERDELTA;
		CCPR1H = (TimerCompare >> 8);
		CCPR1L = (TimerCompare & 0x00ff);

		PIR1bits.CCP1IF = 0;
	}

	//If the interrupt was due to serial data from the remote save it to serial FIFO.  
	if(PIR1bits.RCIF == 1)
		FIFOError = SerialProcess();
}


#pragma interruptlow LowISR

void LowISR(void)
{

}

//Define the interrupt vectors for both low and high interrupts
#pragma code highVector=0x08

void HighVector (void)
{
    _asm goto HighISR _endasm
}

#pragma code						//Return to default code section

#pragma code lowVector=0x18

void LowVector (void)
{
    _asm goto LowISR _endasm
}

#pragma code						//Return to default code section


// Main application entry point.

void main(void)
{
	HalfSecFlag = FALSE;
	VibrationAlarmFlag = FALSE;
	TemperatureAlarmFlag = FALSE;
	FIFOError = FALSE;
	SerialTokenCount = 0;
	SerialMessageIndex = 0;
	CalculatedCRC = 0;
	RecoveredCRC = 0;
	TimerCompare = 0;
	HalfSecCounter = 0;
	RightFrontVibration = 0;
	LeftFrontVibration = 0;
	RightRearVibration = 0;
	LeftRearVibration = 0;
	RightFrontVibrationDisplay = 0;
	LeftFrontVibrationDisplay = 0;
	RightRearVibrationDisplay = 0;
	LeftRearVibrationDisplay = 0;
	
	RightFrontTemperature = 0;
	LeftFrontTemperature = 0;
	RightRearTemperature = 0;
	LeftRearTemperature = 0;
		
	RightFrontTemperatureF = 0;
	LeftFrontTemperatureF = 0;
	RightRearTemperatureF = 0;
	LeftRearTemperatureF = 0;	

	SerialBuffer[0] = 0;
	CRC_value = 0;
	Limit_setting = 0;
	VibrationLimit = 0;
	OverLimitCounter = 0;

	DataValid = FALSE;

	SerialMode = SERIAL_START;
	SerialInit;

	//Sets the IO bits, ADC, and USART
    InitializeBoard();

	Reset_display();
	Write_test_message();

	//Display current board configuration info
	printf("\r\r\r");
	printf(VERSION);


	//We now enter into an infinite loop application tasks are also called as needed.  This 
	//implementation assumes a co-operative multi-taking approach/.

	printf("We are now starting the main application loop\r\n");

    while(1)
    {
		DataProcess();				//Read data from the serial FIFO and decode the RDU packet

		//Display the new data if it was received and is valid
		if(DataValid == TRUE)
		{			
			//The input vibration values are clipped at about 450 so we divide by 41 which will form a number
			//between 0 and 11 which we display as a bar graph
			RightFrontVibrationDisplay = (uint8)ceil(RightFrontVibration/41.0);
			LeftFrontVibrationDisplay = (uint8)ceil(LeftFrontVibration/41.0);
			RightRearVibrationDisplay = (uint8)ceil(RightRearVibration/41.0);
			LeftRearVibrationDisplay = (uint8)ceil(LeftRearVibration/41.0);

			//If temperature is below freezing force to zero
			if(RightFrontTemperature < 0)
				RightFrontTemperature = 0;
			if(LeftFrontTemperature < 0)
				LeftFrontTemperature = 0;
			if(RightRearTemperature < 0)
				RightRearTemperature = 0;
			if(LeftRearTemperature < 0)
				LeftRearTemperature = 0;				

			//Convert temperature to deg F
			RightFrontTemperatureF = (uint8)((float)RightFrontTemperature *0.0625 * 9/5) + 32;
			LeftFrontTemperatureF = (uint8)((float)LeftFrontTemperature *0.0625 * 9/5) + 32;
			RightRearTemperatureF = (uint8)((float)RightRearTemperature *0.0625 * 9/5) + 32;
			LeftRearTemperatureF = (uint8)((float)LeftRearTemperature *0.0625 * 9/5) + 32;
			
			//Read the limit setting pot to determine the driver selected alarm level
			ADCON0 = 0b00000001;		//Select the channel
			ADCON0bits.GO_DONE = 1;		//Start the conversion
	
			while(ADCON0bits.GO_DONE == 1);		//Wait until the conversion is complete
	
			Limit_setting = ((uint16)ADRESH << 8) + ((uint16)ADRESL);
			VibrationLimit = (uint8)((Limit_setting * 12)>>10);

			//Read the toggle mode select switch and display the correct information
			if(MODE_IO == 0)
			{
				//Display the front sensor data
				Write_vibration(RightFrontVibrationDisplay, LeftFrontVibrationDisplay, VibrationLimit, TRUE);	
				Write_temperature(RightFrontTemperatureF, LeftFrontTemperatureF);
			}
			else
			{
				//Display the rear sensor data
				Write_vibration(RightRearVibrationDisplay, LeftRearVibrationDisplay, VibrationLimit, FALSE);	
				Write_temperature(RightRearTemperatureF, LeftRearTemperatureF);
			}

			//Check the current vibration level against the limit.
			//Alarm only if the level exceeds the limit for 5 seconds
			if((RightFrontVibrationDisplay > VibrationLimit) | (LeftFrontVibrationDisplay > VibrationLimit) |
				(RightRearVibrationDisplay > VibrationLimit) | (LeftRearVibrationDisplay > VibrationLimit))
				OverLimitCounter++;
			else
				OverLimitCounter = 0;

			if(OverLimitCounter > 10)				//This is 10 since the RDU sends data every half second
				VibrationAlarmFlag = TRUE;
			else
				VibrationAlarmFlag = FALSE;

			//Set the alarm if the temperature exceeds 160
			if((RightFrontTemperatureF > 160) | (LeftFrontTemperatureF > 160) | 
				(RightRearTemperatureF > 160) | (LeftRearTemperatureF > 160))
				TemperatureAlarmFlag = TRUE;
			else
				TemperatureAlarmFlag = FALSE;

			if((VibrationAlarmFlag == TRUE) | (TemperatureAlarmFlag == TRUE))
				ALARM = 1;
			else
				ALARM = 0;
				
			DataValid = FALSE;

		}		//if(DataValid == TRUE)

		//Flash the one second LED to show that the main loop is alive		
		if(HalfSecFlag == TRUE)
		{
			ONE_SEC_LED ^= 1;		//Flash the LED
			HalfSecFlag = FALSE;
		} 


		//Kick the dog
		_asm CLRWDT _endasm

    }	//while(1)
}		//void main(void)



/**
	Initialize the board by configuring the I/O ports, setting up the ADC, configuring the 
	UART, and enabling the interrupts.

  	@return void

*/
static void InitializeBoard(void)
{
	//The internal oscillator set to 8 MHz with PLL at x4
	OSCCONbits.IRCF0 = 1;
	OSCCONbits.IRCF1 = 1;
	OSCCONbits.IRCF2 = 1;

	OSCTUNEbits.PLLEN = 1;

	//Disable all interrupts until they are configured
    INTCONbits.GIEH = 0;	//Disable all interrupts
    INTCONbits.GIEL = 0;	//Perpheral interrupts disabled

	//Configure the ADC
	ADCON0 = 0b00000001;		//ADC ON
	ADCON1 = 0b00001110;		//AN0 as analog, 0 to 5 V reference
	ADCON2 = 0b10111010;		//Rigth justify, 20 TAD delay, Fosc/32 clock 


	//Configure timer 1 to generate a 1 msec interrupt in conjunction with
	//CPP1.  /8 prescale.  Timer clock is 1 usec (32/4/8)
    T1CONbits.RD16 = 1;
    T1CONbits.T1RUN = 0;
    T1CONbits.T1CKPS1 = 1;
    T1CONbits.T1CKPS0 = 1;
    T1CONbits.T1OSCEN = 0;
    T1CONbits.T1SYNC = 0;
    T1CONbits.TMR1CS = 0;
    T1CONbits.TMR1ON = 0;		//keep timer off for now

    TMR0H = 0;
    TMR0L = 0;

	//Set compare module to generate interrupt on compare and to use timer1 as the
	//clock source
    CCP1CONbits.CCP1M3 = 1;
    CCP1CONbits.CCP1M2 = 0;
    CCP1CONbits.CCP1M1 = 1;
    CCP1CONbits.CCP1M0 = 0;
	T3CONbits.T3CCP1 = 0;
	T3CONbits.T3CCP2 = 0;

	PIE1bits.CCP1IE = 1;		//Enable interrupts from CCP1
	PIR1bits.CCP1IF = 0;		//Clear interrupt flag

	//Set the compare register for the first interval
	TimerCompare = TIMERDELTA;
	CCPR1H = (TimerCompare >> 8);
	CCPR1L = (TimerCompare & 0x00ff);


	//Configure USART
	TRISCbits.TRISC6 = 0;		//USART Tx pin
	TRISCbits.TRISC7 = 1;		//USART Rx pin
    TXSTA = 0b00100000;			//Tx enable, Asyn mode, 8 bits Tx, low baud rate
    RCSTA = 0b10010000;			//Serial port enable, 8 bits Rx, Rx enable
    SPBRG = SPBRG_VAL;

	//Note:  The manual indicates that the UART automatically configures
	//the I/O pin for input when the UART receiver is selected but 
	//this does not look to be the case so we set the pin as an input 
	//to prevent contention.
	TRISC = TRISC | 0b10000000;

	PIE1bits.RCIE = 1;		//Enable interrupts from UART
	PIR1bits.RCIF = 0;		//Clear interrupt flag


	ONE_SEC_LED_TRIS = 0;	//Configure the 1 second LED for output
	ONE_SEC_LED = 1;

	ALARM_TRIS = 0;			//Configure as output
	ALARM = 0;

	RS_IO_TRIS = 0;			//Configure as output
	RS_IO = 0;

	RW_IO_TRIS = 0;			//Configure as output
	RW_IO = 0;

	E_IO_TRIS = 0;			//Configure as output
	E_IO = 0;

	MODE_TRIS = 1;			//Configure as input

	ConfigDataPortOut();	//Configure the display data port as output

	T1CONbits.TMR1ON = 1;		//Start the timer

	// Enable Interrupts
    INTCONbits.GIEH = 1;		//High priority interrupts enabled
	INTCONbits.GIEL = 1;		//Perpheral interrupts enabled
}






/*******************************************************************************************
     Calculate the CRC-16 CCITT of <b>buffer</b> that is <b>length</b> bytes long.

     @param buffer Pointer to data buffer.
     @param length number of bytes in data buffer
	 @param crc starting value

     @return CRC-16 of buffer[0 .. length-1]
*******************************************************************************************/
uint16 sysCRC16 (uint8 *buffer, uint16 length, uint16 crc)
{
	uint16 i, bit, value;

	for (i = 0; i < length; ++i) {
		value = buffer[i];

		for (bit = 0; bit < 8; ++bit) {
			crc ^= (value & 0x01);
			crc = ( crc & 0x01 ) ? ( crc >> 1 ) ^ 0x8408 : ( crc >> 1 );
			value = value >> 1;
		}
	}		// End for (i = 0; i < length; ++i)

   return crc ^ 0xffff;
}





/**

	Process the received data to extract the vibration and temperature information.  The message format is:

	Data>RightMaxVib,LeftMaxVib,CenterMaxVib,RightTemperature,LeftTemperature,Battery_voltage,Battery_current,Battery_Ah,CRC

	@return void

*/
void DataProcess(void)
{
	uint8 SerialData;

	//If data is available in the FIFO process it
	while(SerialIsDataReady() == TRUE)
	{
		SerialData = SerialRead();

		switch(SerialMode)
		{
			case SERIAL_START:
	
				if(SerialData == 'D')
				{
					SerialTokenCount = 0;
					SerialMessageIndex = 0;
					SerialToken[0] = 0;
					SerialToken[1] = 0;
					SerialToken[2] = 0;
					SerialToken[3] = 0;
					SerialToken[4] = 0;
					SerialToken[5] = 0;
					SerialToken[6] = 0;
					SerialToken[7] = 0;
					SerialToken[8] = 0;
	
					SerialBuffer[SerialMessageIndex++] = SerialData;
	
					SerialMode = SERIAL_START_A;
				}
				break;
	
	
			case SERIAL_START_A:
	
				if(SerialData == 'a')
				{
					SerialBuffer[SerialMessageIndex++] = SerialData;
					SerialMode = SERIAL_START_T;
				}
				else
					SerialMode = SERIAL_START;
				break;
	
	
			case SERIAL_START_T:
	
				if(SerialData == 't')
				{
					SerialBuffer[SerialMessageIndex++] = SerialData;
					SerialMode = SERIAL_START_2A;
				}
				else
					SerialMode = SERIAL_START;
				break;
	
	
			case SERIAL_START_2A:
	
				if(SerialData == 'a')
				{
					SerialBuffer[SerialMessageIndex++] = SerialData;
					SerialMode = SERIAL_START_M;
				}
				else
					SerialMode = SERIAL_START;
				break;
	
	
			case SERIAL_START_M:
	
				if(SerialData == '>')
				{
					SerialToken[SerialTokenCount++] = SerialMessageIndex;
					SerialBuffer[SerialMessageIndex++] = SerialData;
					SerialMode = SERIAL_MESSAGE;
				}
				else
					SerialMode = SERIAL_START;
				break;
	
	
			case SERIAL_MESSAGE:
	
				if(SerialData == ',')
					SerialToken[SerialTokenCount++] = SerialMessageIndex;
	
				SerialBuffer[SerialMessageIndex++] = SerialData;
	
				if(SerialTokenCount >= 9)
					SerialMode = SERIAL_CRC;
	
				break;
	
	
			case SERIAL_CRC:

				SerialBuffer[SerialMessageIndex++] = SerialData;
	
				if(SerialData == 0x0d)
				{
					SerialMode = SERIAL_DECODE;
					SerialToken[SerialTokenCount] = SerialMessageIndex - 1;
				}
	
				//If we have found the end of the message (0x0d) we went to proceed directly to the
				//SERIAL_DECODE state without pulling in a new  byte of data
				if(SerialMode != SERIAL_DECODE)
					break;
	
	
			case SERIAL_DECODE:
	
				//C18 does not have a scanf function so we have to decode the delimited string
				//the hard way
				CalculatedCRC = sysCRC16(SerialBuffer,SerialToken[8], 0x12);
				SerialBuffer[SerialToken[9]] = 0;		//Null terminate the CRC sub-string
				RecoveredCRC = atoi(&SerialBuffer[SerialToken[8] + 1]);

				if(CalculatedCRC == (uint16)RecoveredCRC)
				{
					//Replace all the tokens with a null 
					for(SerialTokenCount = 0;SerialTokenCount < 9;SerialTokenCount++)
						SerialBuffer[SerialToken[SerialTokenCount]] = 0;		
	
					RightFrontVibration = (uint16) atoi(&SerialBuffer[SerialToken[0] + 1]);
					LeftFrontVibration = (uint16) atoi(&SerialBuffer[SerialToken[1] + 1]);
					RightRearVibration = (uint16) atoi(&SerialBuffer[SerialToken[2] + 1]);
					LeftRearVibration = (uint16) atoi(&SerialBuffer[SerialToken[3] + 1]);
	
					RightFrontTemperature = atoi(&SerialBuffer[SerialToken[4] + 1]);
					LeftFrontTemperature = atoi(&SerialBuffer[SerialToken[5] + 1]);
					RightRearTemperature = atoi(&SerialBuffer[SerialToken[6] + 1]);
					LeftRearTemperature = atoi(&SerialBuffer[SerialToken[7] + 1]);
					
					DataValid = TRUE;
				}
				else
					printf("CRC error:  %d, %d\r", CalculatedCRC,RecoveredCRC);
		
				SerialMode = SERIAL_START;	
			
				break;

		}	//switch
	}		//while
}

