/**
  ******************************************************************************
  * @file    i2c.c
  * @author  D. Erickson
  * @version 0.0
  * @date    03/13/2011
  * @brief   This file provides a set of functions needed to manage the I2C
  *			 devices for the preamp: the 8574 for the keypad, the MCP23008's for
  * 		 the crosspoint control and the Preamp selection. And the
  *			 TDA7439s. I'ts my first I2C device in 15 years, so bear with me.
  * @history 03/13/11 dje  I2C working. Challenges: PCF8574 had address shifted.
  * 		 Code has no timeout. If no ACK, it just hangs. Meaning that all I2C
  * 	     devices must be present and at the correct addresses
  * 		 Write buffer working, writes multiple bytes. Verified I2C clock speed.
  * 		 No I2C reads yet, but I really don't need them yet. Used LM75 example.
  *
  * 		 03/16/11 dje  Removed DMA control code with #ifdef
  * 		 03/19/11 dje  I2C for MCP23008 and XPT working. Few reworks to XPT board needed
  *			 04/03/11 dje  Fixed bugs in preamp_set() to get all 8 channels working
  ******************************************************************************
  * @copy
  *
  *
  */

/* Includes ------------------------------------------------------------------*/
#include "i2c.h"
#include "main.h"
#include "stm32f10x_conf.h"


//#define I2C_DMA

/** @defgroup STM32_EVAL_I2C_EE_Private_Variables
  * @{
  */

DMA_InitTypeDef   preDMA_InitStructure;

/**
  * @brief  DeInitializes peripherals used by the I2C EEPROM driver.
  * @param  None
  * @retval None
  */
void i2c_DeInit(void)
{
//  i2c_LowLevel_DeInit();
}

/**
  * @brief  Initializes peripherals used by the I2C EEPROM driver.
  * @param  None
  * @retval None
  */
void preamp_i2c_Init(void)
{
  I2C_InitTypeDef  I2C_InitStructure;

  i2c_LowLevel_Init();

  /*!< I2C configuration */
  /* pre_I2C configuration */
  I2C_InitStructure.I2C_Mode = I2C_Mode_I2C;
  I2C_InitStructure.I2C_DutyCycle = I2C_DutyCycle_2;
  I2C_InitStructure.I2C_OwnAddress1 = I2C_SLAVE_ADDRESS7;
  I2C_InitStructure.I2C_Ack = I2C_Ack_Enable;
  I2C_InitStructure.I2C_AcknowledgedAddress = I2C_AcknowledgedAddress_7bit;
  I2C_InitStructure.I2C_ClockSpeed = I2C_SPEED;

  /* pre_I2C Peripheral Enable */
  I2C_Cmd(pre_I2C, ENABLE);
  /* Apply pre_I2C configuration after enabling it */
  I2C_Init(pre_I2C, &I2C_InitStructure);

#ifdef I2C_DMA
  /* Enable the pre_I2C peripheral DMA requests */
  I2C_DMACmd(pre_I2C, ENABLE);
#endif
}
/**
  * @brief  Initializes peripherals used by the I2C EEPROM driver.
  * @param  None
  * @retval None
  */


void i2c_LowLevel_Init(void)
{
  GPIO_InitTypeDef  GPIO_InitStructure;
//  NVIC_InitTypeDef NVIC_InitStructure;

  /*!< pre_I2C_SCL_GPIO_CLK and pre_I2C_SDA_GPIO_CLK Periph clock enable */
  RCC_APB2PeriphClockCmd(pre_I2C_SCL_GPIO_CLK | pre_I2C_SDA_GPIO_CLK, ENABLE);

  /*!< pre_I2C Periph clock enable */
  RCC_APB1PeriphClockCmd(pre_I2C_CLK, ENABLE);

  /*!< GPIO configuration */
  /*!< Configure pre_I2C pins: SCL */
  GPIO_InitStructure.GPIO_Pin = pre_I2C_SCL_PIN;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_OD;
  GPIO_Init(pre_I2C_SCL_GPIO_PORT, &GPIO_InitStructure);

  /*!< Configure pre_I2C pins: SDA */
  GPIO_InitStructure.GPIO_Pin = pre_I2C_SDA_PIN;
  GPIO_Init(pre_I2C_SDA_GPIO_PORT, &GPIO_InitStructure);

#ifdef I2C_DMA
  /* Configure and enable I2C DMA TX Channel interrupt */
  NVIC_InitStructure.NVIC_IRQChannel = pre_I2C_DMA_TX_IRQn;
  NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = pre_I2C_DMA_PREPRIO;
  NVIC_InitStructure.NVIC_IRQChannelSubPriority = pre_I2C_DMA_SUBPRIO;
  NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
  NVIC_Init(&NVIC_InitStructure);

  /* Configure and enable I2C DMA RX Channel interrupt */
  NVIC_InitStructure.NVIC_IRQChannel = pre_I2C_DMA_RX_IRQn;
  NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = pre_I2C_DMA_PREPRIO;
  NVIC_InitStructure.NVIC_IRQChannelSubPriority = pre_I2C_DMA_SUBPRIO;
  NVIC_Init(&NVIC_InitStructure);

  /*!< I2C DMA TX and RX channels configuration */
  /* Enable the DMA clock */
  RCC_AHBPeriphClockCmd(pre_I2C_DMA_CLK, ENABLE);

  /* I2C TX DMA Channel configuration */
  DMA_DeInit(pre_I2C_DMA_CHANNEL_TX);
  preDMA_InitStructure.DMA_PeripheralBaseAddr = (uint32_t)pre_I2C_DR_Address;
  preDMA_InitStructure.DMA_MemoryBaseAddr = (uint32_t)0;   /* This parameter will be configured durig communication */
  preDMA_InitStructure.DMA_DIR = DMA_DIR_PeripheralDST;    /* This parameter will be configured durig communication */
  preDMA_InitStructure.DMA_BufferSize = 0xFFFF;            /* This parameter will be configured durig communication */
  preDMA_InitStructure.DMA_PeripheralInc = DMA_PeripheralInc_Disable;
  preDMA_InitStructure.DMA_MemoryInc = DMA_MemoryInc_Enable;
  preDMA_InitStructure.DMA_PeripheralDataSize = DMA_MemoryDataSize_Byte;
  preDMA_InitStructure.DMA_MemoryDataSize = DMA_MemoryDataSize_Byte;
  preDMA_InitStructure.DMA_Mode = DMA_Mode_Normal;
  preDMA_InitStructure.DMA_Priority = DMA_Priority_VeryHigh;
  preDMA_InitStructure.DMA_M2M = DMA_M2M_Disable;
  DMA_Init(pre_I2C_DMA_CHANNEL_TX, &preDMA_InitStructure);

  /* I2C RX DMA Channel configuration */
  DMA_DeInit(pre_I2C_DMA_CHANNEL_RX);
  DMA_Init(pre_I2C_DMA_CHANNEL_RX, &preDMA_InitStructure);

  /* Enable the DMA Channels Interrupts */
  DMA_ITConfig(pre_I2C_DMA_CHANNEL_TX, DMA_IT_TC, ENABLE);
  DMA_ITConfig(pre_I2C_DMA_CHANNEL_RX, DMA_IT_TC, ENABLE);
#endif
}



/**
  * @brief  Writes 7 bit address , then *buffer to I2C device
  * @param  buffer : pointer to the buffer  containing the data
  * 		to be written
  * @param  count: buffer size, number of bytes sent after addr
  * @retval None
  * 		Currently will hang if no device is present
  */
void i2c_Write(uint8_t addr, uint8_t * buffer, uint8_t count)
{
  uint8_t i;
	/*!< Send START condition */
  I2C_GenerateSTART(pre_I2C, ENABLE);
  /*!< Test on EV5 and clear it */
  while(!I2C_CheckEvent(pre_I2C, I2C_EVENT_MASTER_MODE_SELECT))
  {
  }
  /*!< Send device address for write */
  I2C_Send7bitAddress(pre_I2C, addr, I2C_Direction_Transmitter);
  /*!< Test on EV6 and clear it */
  while(!I2C_CheckEvent(pre_I2C, I2C_EVENT_MASTER_TRANSMITTER_MODE_SELECTED))
  {
  }
  for(i = 0 ; i < count; i++ ) 	/* Send 'count' bytes */
  {
	  I2C_SendData(pre_I2C, *buffer++);
  /*!< Test on EV8 and clear it */
	  while(!I2C_CheckEvent(pre_I2C, I2C_EVENT_MASTER_BYTE_TRANSMITTED))
	  {
	  }
  }
  /*!< Send STOP condition */
  I2C_GenerateSTOP(pre_I2C, ENABLE);
}

/* To set the crosspoint:
	Turn the old source OFF
	Turn the new source ON
To change a switch:
	Set up data, zone and source
	Set the strobe
	Clear the strobe */

void xpt_set(uint8_t zone, uint8_t old, uint8_t new)
{
	uint8_t i2c_buf[8];

	zone &= XPT_ZONE; 				/* Mask the Zone bits */

	old = old << 3;
	old  &= XPT_SRC;				/* Mask the old and new channel bits */
	new = new << 3;
	new  &= XPT_SRC;

	i2c_buf[0] = M23008_GPIO;		/* Set register to GPIO */
	i2c_buf[1] = XTB_DOFF | XTB_ST_OFF| old | zone;	/* Old Channel OFF */
	i2c_buf[2] = XTB_DOFF | XTB_ST_ON | old | zone;
	i2c_buf[3] = XTB_DOFF | XTB_ST_OFF| old | zone;
	i2c_buf[4] = XTB_DON  | XTB_ST_OFF| new | zone;	/* New Channel ON */
	i2c_buf[5] = XTB_DON  | XTB_ST_ON | new | zone;
	i2c_buf[6] = XTB_DON  | XTB_ST_OFF| new | zone;

	i2c_Write(I2C_XPT_ADDR, i2c_buf, 7);

//	preamp[zone].source = new >> 0;
}

/* Set all crosspoint switches off except input 0 */
void xpt_init()
{
	uint8_t chan, old;
	uint8_t input = 0;						/* Set all channels to input 0 */
	for (chan = 0; chan < 8; chan++)
	{
		for (old = 0; old < 8; old++)
		{
			xpt_set(chan, old, input);		/* Turn off all switches, set all channels to 0 */
		}
		preamp[chan].source = input;
	}
}
/* Simple init of all channels to default settings. */

void preamp_init(void)
{
	uint8_t chan;
	for (chan = 0; chan < ZONE_MAX+1; chan++)
	{
		preamp_set(chan);
	}
}


/* First set up the channel select to access a specific preamp,  as follows:
 * For zones 0-3:
 * 		Set I2C_PRESEL1 bit 2 to 1 to inhibit this board.
 * 		Set I2C_PRESEL0 bit 2 to 0 (no inhibit) and bits 1:0 to (zone & 0x03)
 * For zones 4-7:
 * 		Set I2C_PRESEL0 bit 2 to 1 to inhibit this board.
 * 		Set I2C_PRESEL1 bit 2 to 0 (no inhibit) and bits 1:0 to (zone & 0x03)
 * Then use I2C_PREAMP to address the preamp
 * For now, set all the registers from the big preamp structure. If this is too noisy or
 * uses too much time, then this can be optimized to set only modified settings.
*/
void preamp_set(uint8_t chan)
{
	uint8_t i2c_buf[8];
	uint8_t vr, br, mr, tr, spl, spr, mu, vol;
//	uint8_t s;
	int8_t bal, bal_vol;
//	char buf1[32];

if (chan <= ZONE_MAX) 						/* Only control existing channels */
{
/* Select the preamp channel  */
	if (chan < 4)							/* Zones 0-3 */
	{
#ifdef CHAN_8_BOARD
		i2c_buf[0] = M23008_GPIO;			/* Turn off I2C_PRESEL1 */
		i2c_buf[1] = 0x04;					/* Set bit 2 to inhibit 4052 on board 1 */
		i2c_Write(I2C_PRESEL1, i2c_buf, 2);
#endif
		i2c_buf[0] = M23008_GPIO;			/* Turn on I2CPRESEL0 */
		i2c_buf[1] = chan & 0x03;			/* Set select code on board 0 */
		i2c_Write(I2C_PRESEL0, i2c_buf, 2);
	}
	else		/* (chan >= 4) Won't get here unless 8_CHAN */
	{
		i2c_buf[0] = M23008_GPIO;			/* Turn off I2C_PRESEL0 */
		i2c_buf[1] = 0x04;					/* Set bit 2 to inhibit 4052 on board 0 */
		i2c_Write(I2C_PRESEL0, i2c_buf, 2);

		i2c_buf[0] = M23008_GPIO;			/* Turn on I2CPRESEL1 */
		i2c_buf[1] = chan & 0x03;			/* Set select code on board 1 */
		i2c_Write(I2C_PRESEL1, i2c_buf, 2);
	}

//	s   = preamp[chan].source;
	vr  = (uint8_t) -preamp[chan].vol;
	vol = vr;
	br  = toneScale(preamp[chan].bass);
	mr  = toneScale(preamp[chan].mid);
	tr  = toneScale(preamp[chan].treb);
	bal = preamp[chan].bal;
	mu  = preamp[chan].mute;

//	sprintf(buf1,"tr=%d\n\r", (int)tr);
//	putString(buf1);

	/* Calculate balance settings for each speaker */
	/* Volume needs more range, so if volume < 46, adjust balance down for both speakers
	 *
	 */
	vol = vr;
	if (vr >=  VOL_CHIP_MIN)
	{
		bal_vol = vr - VOL_CHIP_MIN;
		vr = VOL_CHIP_MIN;
	}
	else
	{
		bal_vol = 0;
	}

	if (bal < 0)
	{
		spl = 0; 				/* Bal < 0, Left = 0dB, less Right */
		spr = (uint8_t)-bal;
	}
	else
	{
		spr = 0; 				/* Bal >= 0, Right = 0dB, less Left */
		spl = bal;
	}

	spl += bal_vol;				/* add in bal_vol adjustment */
	spr += bal_vol;

	if ((mu == 1) || (vol == -VOL_MIN))		/* Check muting, set vol to mute value */
	{
		vr = TDA7439_MUTE;
	}


	TDA7439_write(TDA7439_INSEL, TDA7439_IN1);	/* Select input 1 (can be init only)*/
	TDA7439_write(TDA7439_INGAIN,0);			/* Set ingain = 0dB */
	TDA7439_write(TDA7439_VOL,  vr);			/* Set Volume */
	TDA7439_write(TDA7439_BASS, br); 			/* Set Bass */
	TDA7439_write(TDA7439_MID,  mr); 			/* Set Midrange */
	TDA7439_write(TDA7439_TREB, tr); 			/* Set Treble */
	TDA7439_write(TDA7439_SPL, spl); 			/* Set Left Bal */
	TDA7439_write(TDA7439_SPR, spr); 			/* Set Right Bal*/

// Init that works
//	TDA7439_write(TDA7439_INSEL, 3);	/* Select input 1 (can be init only)*/
//	TDA7439_write(TDA7439_INGAIN, 0);	/* Set ingain = 0dB */
//	TDA7439_write(TDA7439_VOL, 0);		/* Set Volume */
//	TDA7439_write(TDA7439_BASS, 0); 	/* Set Bass */
//	TDA7439_write(TDA7439_BASS, 0); 	/* Set Midrange */
//	TDA7439_write(TDA7439_TREB, 0); 	/* Set Treble */
//	TDA7439_write(TDA7439_SPL, 0); 		/* Set Left Bal */
//	TDA7439_write(TDA7439_SPR, 0); 		/* Set Right Bal*/

}

} /* End of preamp_set */

/* Set up an MPC23008 to all outputs */
void MCP23008_init(uint8_t device)
{
	uint8_t i2c_buf[4];

	i2c_buf[0] = M23008_IOCON;		/* Set IOCON to disable auto inc. bit 5  */
	i2c_buf[1] = 0x20;
	i2c_Write(device, i2c_buf, 2);

	i2c_buf[0] = M23008_DIR;		/* Set the direction bits to OUTPUT */
	i2c_buf[1] = 0x00;
	i2c_Write(device, i2c_buf, 2);
}

void TDA7439_write(uint8_t reg, uint8_t data)
{
	uint8_t i2c_buf[4];

	i2c_buf[0] = reg;		/* Set register to INSEL */
	i2c_buf[1] = data;
	i2c_Write(I2C_TDA7439, i2c_buf, 2);
}

/* Crazy tone math used in bass, mid and treb of TDA7439
 *  -14dB = 0, -2dB = 6, 0dB = 7, 2dB = 14, 14dB = 8 */
uint8_t toneScale(int8_t tone)
{
	uint8_t reg;

	if (tone <  0)
		{
		reg = (uint8_t)(-tone);
		reg = reg / 2;
		reg = 7 - reg;
		}
	if (tone == 0) reg = 7;
	if (tone >  0) reg = (uint8_t)(15 - ((uint8_t)(tone) / 2));
	return reg;
}
