/**
  ******************************************************************************
  * @file    keypad.c
  * @author  D. Erickson
  * @version 0.0
  * @date    03/13/2011
  * @brief   This file reads all key input controls and translates the commands.
  *
  * @history
  * 	03/19/11 dje  Just started
  *		03/20/11 dje  Keypad implemented
  *		03/21/11 dje  IR processing implemented: needs more.
  *		03/30/11 dje  Fixed keypad delay and repeat timer bug: keypad_timer was
  *		incremented here, should be only in time tick interrupt.
  *		04/01/11 dje  Added simple IR filter
  *		04/10/11 dje  Added Remote keypads
  *		04/12/11 dje  Added encoder. Works great!
  *		07/24/11 dje  Structure based keypads working on FPKEY, repeat is selective
  *		07/24/11 dje  Remote keypads are working with inKey[]
  *
  ******************************************************************************
  * @copy
  *
  *
  */

#include "keypad.h"
#include "main.h"
#include "i2c.h"
#include "stm32f10x_conf.h"

#include <stdio.h>

extern inKeyTypeDef inKey[];
extern irTypeDef ir;


/* Voltage limits(in ADC counts) For 7 x 1K resistors, 4.7K pull-up, 1K in series.
 * limits are +/- 60 counts = +/- 1.5% of full scale 4096.
 * These were calculated in Excel.
 * These limits are a bit tight and can be loosened by
 * changing the higher button resistors to larger values */
const uint16_t remkey_limit[]= {
658,	778,     /* Key 0 */
1162,	1282,    /* Key 1 */
1535,	1655,    /* Key 2 */
1823,	1943,    /* Key 3 */
2051,	2171,    /* Key 4 */
2236,	2356,    /* Key 5 */
2390,	2510,    /* Key 6 */
2520,	2640     /* Key 7 */
};

/* sugested Key Orientation for remote analog keypads:
    7: V	3: 0
    6: v	2: 0
    5: l	1: r
    4: n	0: q
*/

const char remkeyMap[]= {
'q',		/* Mute */
'r',		/* Balance  R */
'0',		/* unused */
'0',		/* unused */
'n',		/* Select next source */
'l',		/* Balance  R */
'v',		/* Vol Down */
'V',		/* Vol Up */
};


/* 128 Sony InfraRed IR codes */
const char irKeyMap[]= {
    0,      /* 0  0x00 */
    0,      /* 1  0x01 */
    0,      /* 2  0x02 */
    0,      /* 3  0x03 */
    0,      /* 4  0x04 */
    0,      /* 5  0x05 */
    0,      /* 6  0x06 */
    0,      /* 7  0x07 */
    0,      /* 8  0x08 */
    0,      /* 9  0x09 */
    0,      /* 10 0x0A */
    0,      /* 11 0x0B */
    0,      /* 12 0x0C */
    0,      /* 13 0x0D */
    0,      /* 14 0x0E */
    0,      /* 15 0x0F */

    0,    	/* 16 0x10 */
    0,    	/* 17 0x11 - 1FA 1441 Key 1 */
    'V',    /* 18 0x12 - Key VOL^ */
    'v',    /* 19 0x13 - Key VOLv */
    'q',    /* 20 0x14 - 1FA 1441 Key Mute */
    0,      /* 21 0x15 */
    0,      /* 22 0x16 */
    0,      /* 23 0x17 */
    0,      /* 24 0x18 */
    0,      /* 25 0x19 */
    0,      /* 26 0x1A */
    0,      /* 27 0x1B */
    0,      /* 28 0x1C */
    '5',    /* 29 0x1D - IFA 1441 Key 5 */
    '9',    /* 30 0x1E - IFA 1441 Key 9 */
    0,      /* 31 0x1F */

    '7',    /* 32 0x20 - IFA 1441 Key 7 */
    '1',    /* 33 0x21 - 1FA 1441 Key 1 */
    '4',    /* 34 0x22 - IFA 1441 Key 4 */
    0,      /* 35 0x23 */
    0,      /* 36 0x24 */
    '3',    /* 37 0x25 - IFA 1441 Key 3 */
    0,      /* 38 0x26 */
    0,      /* 39 0x27 */
    0,      /* 40 0x28 */
    0,      /* 41 0x29 */
    0,      /* 42 0x2A */
    0,      /* 43 0x2B */
    0,      /* 44 0x2C */
    0,      /* 45 0x2D */
    0,      /* 46 0x2E */
    0,      /* 47 0x2F */

    0,      /* 48 0x30 */
    0,      /* 49 0x31 */
    0,      /* 50 0x32 */
    0,      /* 51 0x33 */
    0,      /* 52 0x34 */
    0,      /* 53 0x35 */
    0,      /* 54 0x36 */
    0,      /* 55 0x37 */
    0,      /* 56 0x38 */
    0,      /* 57 0x39 */
    0,      /* 58 0x3A */
    0,      /* 59 0x3B */
    0,      /* 60 0x3C */
    0,      /* 61 0x3D */
    0,      /* 62 0x3E */
    0,      /* 63 0x3F */

    0,      /* 64 0x40 */
    0,      /* 65 0x41 */
    0,      /* 66 0x42 */
    0,      /* 67 0x43 */
    '8',    /* 68 0x44 IFA 1441 Key 8*/
    0,      /* 69 0x45 */
    0,      /* 70 0x46 */
    0,      /* 71 0x47 */
    0,      /* 72 0x48 */
    '9',    /* 73 0x49 - Old 1FA Key 9 */
    0,      /* 74 0x4A */
    0,      /* 75 0x4B */
    0,      /* 76 0x4C */
    0,      /* 77 0x4D */
    0,      /* 78 0x4E */
    0,      /* 79 0x4F */

    0,      /* 80 0x50 */
    0,      /* 81 0x51 */
    0,      /* 82 0x52 */
    0,      /* 83 0x53 */
    0,      /* 84 0x54 */
    0,      /* 85 0x55 */
    0,      /* 86 0x56 */
    0,      /* 87 0x57 */
    0,      /* 88 0x58 */
    0,      /* 89 0x59 */
    0,      /* 90 0x5A */
    0,      /* 91 0x5B */
    0,      /* 92 0x5C */
    0,      /* 93 0x5D */
    0,      /* 94 0x5E */
    0,      /* 95 0x5F */

    0,      /* 96 0x60 */
    0,      /* 97 0x61 */
    0,      /* 98 0x62 */
    0,      /* 99 0x63 */
    0,      /* 100 0x64 */
    0,      /* 101 0x65 */
    0,      /* 102 0x66 */
    0,      /* 103 0x67 */
    0,      /* 104 0x68 */
    '8',    /* 105 0x69 - 1FA 1441 Key 8 */
    '6',    /* 106 0x6A - 1FA 1441 Key 6 */
    0,      /* 107 0x6B */
    0,      /* 108 0x6C */
    0,      /* 109 0x6D */
    0,      /* 110 0x6E */
    0,      /* 111 0x6F */

    0,      /* 112 0x70 */
    0,      /* 113 0x71 */
    '0',    /* 114 0x72 - 1FA 1441 Key 0 */
    0,      /* 115 0x73 */
    0,      /* 116 0x74 */
    0,      /* 117 0x75 */
    0,      /* 118 0x76 */
    0,      /* 119 0x77 */
    0,      /* 120 0x78 */
    0,      /* 121 0x79 */
    0,      /* 122 0x7A */
    0,      /* 123 0x7B */
    0,      /* 124 0x7C */
    '2',    /* 125 0x7D - 1FA 1441 Key 2*/
    0,      /* 126 0x7E */
    0       /* 127 0x7F*/

    };

/* Front Panel Key mapping */
const char FPKeyMap[]= {
    '1',    /* 0  In 0 */
    '2',    /* 1  In 1 */
    '3',    /* 2  In 2 */
    '4',    /* 3  In 3 */
    '5',    /* 4  In 4 */
    '6',    /* 5  In 5 */
    '7',    /* 6  In 6 */
    '8',    /* 7  In 7*/

    'V',    /* 8  Vol  up */
    'r',    /* 9  Bal  r  */
    'B',    /* 10 Bass up */
    'M',    /* 11 Mid up */
    'T',	/* 12 Treble up*/
    'q',    /* 13 Mute*/
    0,	    /* 14 */
    'Q',    /* 15 Mute All*/

    'v',    /* 16 Vol  dn */
    'l',    /* 17 Bal  l  */
    'b',    /* 18 Bass dn */
    'm',    /* 19 Mid +dn*/
    't',    /* 20 Treb dn*/
    'L', 	/* 21 Loudness */
    0,	    /* 22 */
    0,		/* 23 */

    '!',    /* 24 */
    '@',    /* 25 */
    '#',    /* 26 */
    '$',    /* 27 */
    '%',    /* 28 */
    '^',    /* 29 */
    '&',    /* 30 */
    '*',    /* 31 */
    };


/* Unused
const unsigned char inchanMap[] =
{
    1,	    // 0  is IR 1
    2,	    // 1  is Front Panel Keypad
    0,	    // 2
    0,	    // 3
    0,	    // 4
    0	    // 5
};
*/

/* These are the transition tables for the various encoder knobs used.
Since I don't do anything specific with illegal (ILL) codes, they are all 0 (NADA)  */

#ifdef ENC4PPCLICK
/* For encoders that generate a full cycle every click such as HP Optical */
 * One pulse per click Only transitions to state 2 (11) are used. */
const  uint8_t encoder_logic[]= {
/*old new      action
---------------------- */
/* 00 00 */    NADA,
/* 00 01 */    NADA,
/* 00 10 */    NADA,
/* 00 11 */    NADA,
/* 01 00 */    NADA,
/* 01 01 */    NADA,
/* 01 10 */    NADA,
/* 01 11 */    INC,

/* 10 00 */    NADA,
/* 10 01 */    NADA,
/* 10 10 */    NADA,
/* 10 11 */    NADA,
/* 11 00 */    NADA,
/* 11 01 */    DEC,
/* 11 10 */    NADA,
/* 11 11 */    NADA
};
#endif

#ifdef ENC1PPCLICK
/* For encoders that generate one transition per click such as most mechanical encoders.
 * Note that mechanical encoders need pull up resistors and should also have R-C filters
 */
const  uint8_t encoder_logic[]= {
/*old new      action
---------------------- */
/* 00 00 */    NADA,
/* 00 01 */    INC,
/* 00 10 */    DEC,
/* 00 11 */    ILL,
/* 01 00 */    DEC,
/* 01 01 */    NADA,
/* 01 10 */    ILL,
/* 01 11 */    INC,

/* 10 00 */    INC,
/* 10 01 */    ILL,
/* 10 10 */    NADA,
/* 10 11 */    DEC,
/* 11 00 */    ILL,
/* 11 01 */    DEC,
/* 11 10 */    INC,
/* 11 11 */    NADA
};
#endif



/* Structure based keypad processor using inKey[]
 * 7/24/11 Got rid of the statics and globals, replaced with big struct.
 * Process a keypad: Look up the key in the correct table,
 * time the key down, key debounce, and key repeat
 * Inputs:
 * 		inKey.rawKey
 * 		inKey.rawFlag
 *
 * inKey[]repeat_timer is incremented in the 1ms time tick interrupt
 * returns the key in inKey[].keyCode and sets flag inKey[].keyFlag
 * Only repeats on keys in isRepeat()
 *
 * Encoder, UART don't need this processing.
 * IR needs it, but after IR filtered
 *
 * This could be called from an interrupt routine, every 5-10ms to provide
 * more accurate timing than main(), but as long as main() doesn't have major
 * delays, it works fine.
 * */
uint8_t key_proc(uint8_t chan)
{
//	char buf1[32];
	inKey[chan].sendFlag = 0;

	if (chan == FPKEY) 				/* Front Panel Keypad */
		getKeypad();
	else if (chan == IRKEY )		/* IR keypad */
	{
		ir_proc();					/* should be called by interrupt for better timing  */
	}
	else if (chan >= REMKEY0 )		/* Remote keypads */
	{}								/* No action, the interrupt handled it. */

	if (inKey[chan].rawFlag)		/* Key down, look up the key code in appropriate table.  */
	{
		if      (chan == FPKEY)
			inKey[chan].keyCode = FPKeyMap[inKey[chan].rawCode];
		else if (chan == IRKEY)
			inKey[chan].keyCode = irKeyMap[inKey[chan].rawCode];
		else if (chan >= REMKEY0)
			inKey[chan].keyCode = remkeyMap[inKey[chan].rawCode];

//		sprintf(buf1,"chan=%d key=%c \n\r", chan, inKey[chan].keyCode); /* Debug: Display key */
//		putString(buf1);

		if (inKey[chan].keyCode != inKey[chan].oldKey)		/* If key changed, first time. Start timer */
		{
			 inKey[chan].repeatTimer = 0;					/* Reset timer */
			 inKey[chan].state = 0;
			 inKey[chan].oldKey = inKey[chan].keyCode;		/* make it the same key */
			 isRepeat(chan); 								/* Should key repeat? Get inChan[].repeat */
		}
		else												/* Key is same one, do debounce delay, repeat */
		{
			if (inKey[chan].state == 0)
			{
				if (inKey[chan].repeatTimer >= KEY_DEBOUNCE)/* Wait for debounce time */
				{
					inKey[chan].state = 1;					/* Debounce complete, send code */
					inKey[chan].repeatTimer = 0;			/* reset timer */
					inKey[chan].sendFlag = 1;
				}
			}
			else if (inKey[chan].state == 1)
			{
				if (inKey[chan].repeatTimer >= KEY_DELAY) 	/* Wait for delay before repeat time */
				{
					inKey[chan].state = 2;					/* Repeat time complete, send code */
					inKey[chan].repeatTimer = 0;
					if (inKey[chan].repeat) inKey[chan].sendFlag = 1;
				}
			}
			else if (inKey[chan].state >= 2)
			{
				if (inKey[chan].repeatTimer >= KEY_REPEAT) 	/* Wait for repeat time */
				{
					inKey[chan].state = 2;					/* Repeat time complete, send code */
					inKey[chan].repeatTimer = 0;
					if (inKey[chan].repeat) inKey[chan].sendFlag = 1;
				}
			}
		}
	}
	else 										/* No key down, reset timer */
	{
		 inKey[chan].repeatTimer = 0;
		 inKey[chan].oldKey = 0;
		 inKey[chan].state = 0;
	}

	if (inKey[chan].sendFlag)
	{
		inKey[chan].zoneTimer = ZONE_TIMEOUT;		/* Restart the zone timeout */
		return inKey[chan].keyCode;
	}
	else
	{
		return 0;
	}
}

/* Scan the 8x4 keypad one column at a time. Encode the rows,
 * stop when the first key is detected.
 * The 8 columns are driven by a I2C PCA8574. The 4 rows go to 4 pull-up resistors
 * and 4 GPIO pins.
 * Put the key in inKey[] and set the flag */
void getKeypad(void)
{
	uint8_t chan = FPKEY;						/* FPKEYPAD */
	uint8_t i2c_buf[4];
	uint16_t row;
	uint16_t code;
	uint8_t col;
	uint8_t i;

	inKey[chan].rawFlag = 0;				/* Clear flag first */

	col = 1;
	for (i = 0; i< 8; i++)					/* Scan the columns */
	{
		i2c_buf[0] = ~col;					/* Drive one column LOW at a time.  */
		i2c_Write(I2C_KPAD_ADDR, i2c_buf, 1);

		row = GPIOB->IDR >> 12;				/* Rows on GPIO bits 15:12 */
		if (row != 0x000F)					/* Are any bits low? */
		{
			if (~row & 0x0001)	code = 0;	/* Bit 0 low? */
			if (~row & 0x0002)	code = 1;	/* Bit 1 low? */
			if (~row & 0x0004)	code = 2;	/* Bit 2 low? */
			if (~row & 0x0008)	code = 3;	/* Bit 3 low? */
			code = i | (code << 3);			/* Col = 3 LSBs, row = 2 MSBs */
			inKey[chan].rawCode = code;		/* Exit here if key detected */
			inKey[chan].rawFlag = 1;		/* Set Flag */
			/* could exit here */
		}
		col = col << 1;						/* Shift to the next column */
	}
}

/* Debug: Display all the IR pulse times.
 * Note, only valid times are collected and displayed, so it's not a great tool.  */
void ir_Test(void)
{
	int i;
	char buf1[64];

	  for(i=0; i<13; i++)
	  {
		  sprintf(buf1,"%d ", irTest[i]);
		  putString(buf1);
	  }
	  sprintf(buf1,"\n\r");
	  putString(buf1);
}


/* ir_proc() processing for Sony 12 bit IR codes
 * -- strips off and compares the device code (5 MSBs)
 * -- look up the IR key is in key_proc
 * -- Filter: make sure there are 2 in a row and that they come within 100mS
 * -- If more come, key is held down
 * -- delay and repeat timers are here
 * Currently only a crude filter / repeat, and no timer: sends a key on every other IR code.
 * Inputs from capture:
 * 		ir.capFlag
 * 		ir.capCode
 * Outputs to keyProc():
 * 		inKey[IRKEY].rawKey
 * 		inKey[IRKEY].rawFlag
 *
 * Note: needs to not change inKey[].rawKey and inKey[].rawFlag unless IR_TIMEOUT
 */
void ir_proc(void)
{
	uint16_t keycode;
//	char buf1[32];
	uint8_t send_flag;

	send_flag = inKey[IRKEY].rawFlag;	/* Default to previous rawFlag */
	keycode = inKey[IRKEY].rawCode;		/* Default to rawCode */

	/* First check to see if TIMEOUT is exceeded. Check free running timer against the previous capture.
	 *  If it has, clear the state machine */
	if ((ir.timer - ir.oldCapture) > IR_TIMEOUT)
	{
		ir.filterState = 0;
		send_flag = 0;
	}

	if (ir.capFlag)							/* From capture interrupt: valid Sony IR code */
	{
//		ir_Test();
		ir.capFlag = 0;						/* Clear the flag */
		keycode = ir.capCode;				/* Get IR code  */


//		sprintf(buf1,"ir_raw = 0x%x\n\r", keycode); /* Debug IR: Raw code */
//		putString(buf1);
		/* Check for valid Sony device code, bits 11:7 = 0x0800, 16d, Sony stereo */
		if((keycode & 0x0F80) == 0x0800)
		{
			keycode &= 0x007F; 				/* Mask off 7 LSBs to create key code*/

			/* Send after 2 of the same code and no timeout. */
			if ((keycode == ir.oldCode) & (ir.time < IR_TIMEOUT))	/* Same key, no timeout */
			{
				if (ir.filterState == 0)		/* First valid code, don't send */
				{
					ir.filterState ++;
					send_flag = 0;
				}
				else if (ir.filterState == 1)	/* Send after second */
				{
					ir.filterState ++;
					send_flag = 1;
				}
				else if (ir.filterState >= 2)	/* Send after subsequent, but hold state at 2 */
				{
					ir.filterState = 2;
					send_flag = 1;
				}
			}
			else								/* new key */
			{
				ir.filterState = 0;
				send_flag = 0;
			}
			ir.oldCode = keycode;

		}

		else	/* If code but not valid Sony device, hold previous flag and code */
		{
			send_flag = inKey[IRKEY].rawFlag;				/* Default to previous rawFlag */
			keycode = inKey[IRKEY].rawCode & 0x007F ;		/* Default to previous rawCode */
		}
	}

	if (send_flag)
	{
		inKey[IRKEY].rawCode = keycode;
		inKey[IRKEY].rawFlag = 1;
	}
	else
	{
		inKey[IRKEY].rawFlag = 0;
	}
}

/* getRemKey()
 *
 * This is the interrupt routine for the remote keypads, ADC conversion and processing.
 * These keypads each consist of seven 1K 1% resistors and eight switches. These are wired to
 * GND and are polarity independent. When a button is pressed, a resistance from
 * 0.0 to 7.0K is presented. On the preamp CPU are 4.7K pull-up resistors,
 * an additional 1K series resistor for protection and a .1uF filter cap.
 * These allow the keypad resistance  * to be measured.
 * The high and low threshold values for each button are calculated in an Excel spreadsheet
 * and entered in remkey_limit[].
 *
 * This routine performs a single ADC conversion every 1ms time tick. It puts the previous
 * ADC conversion in adc_data[] and then sets up the next conversion. With 1ms between
 * conversions, no wait for conversion complete is needed. 7 remote channels (typically
 * the maximum needed since the main channel, 0, uses the keypad, take
 * 7ms each, plenty fast for key scanning.
 *
 * It processes only the current channel.
 *
 * Then the key is sensed: When the voltage is below the upper threshold of the highest
 * resistance key, then some key is pressed, or there is noise on the line, or the
 * voltage is transition-ing between keys.
 * Then a linear search for the key is done by checking if the voltage is between
 * the high and low thresholds in remkey_limit[] for each of the keys.
 * If the key is valid, look up the command. If not, return a 0
 *
 * Debounce, key delay and repeat are done in key_proc()
 *
 * The indexing through the ADC channels should be through a look-up-table since
 * there are not 7 consecutive ADC input pins available. Currently only 3 keypads are
 * implemented and these are consecutive. To add more, build an index table.
 */
void getRemKey(void)
{
	uint8_t i, j, chan;

	/* Don't need to check ADC ready since it has been 1 millisecond */
	adc_data[adc_index] = ADC1->DR;			/* Get previous ADC channel, 0 based*/
	i = adc_index;							/* Get index */

	adc_index ++;
	if (adc_index >= REMKEY_NUM)			/* If the counter rolls over */
	{
		adc_index = 0;
	}

	chan = REMKEY0 + i;						/* Current channel */

	adc_chan ++;	/* Set up the next one. ADC channels for keypads are not necessarily in order. Needs a LUT */
	if (adc_chan > ADC_MAX)					/* If the counter rolls over */
	{
		adc_chan = ADC_MIN;
	}
											/* Fire the next channel ADC */
	ADC_RegularChannelConfig(ADC1, adc_chan, 1, ADC_SampleTime_55Cycles5);
	ADC_SoftwareStartConvCmd(ADC1, ENABLE);

	/* Process adc_data[i]. First see if some key is pressed, then search for it. */
	inKey[chan].rawFlag = 0;				/* Start with invalid */
	if(adc_data[i] > remkey_limit[15])		/* This is the top key threshold: no key pressed */
	{
											/* Do nothing, rawFlag already = 0 */
	}
	else									/* Search for valid key in the limit table*/
	{
		inKey[chan].rawFlag = 0;
		for(j = 0; j < 8; j++)
		{									/* See if the code is between HI and LOW limits */
			if((adc_data[i] >= remkey_limit[j*2]) && (adc_data[i] <= remkey_limit[(j*2)+1]))
			{
				inKey[chan].rawCode = j;	/* Found a valid voltage */
				inKey[chan].rawFlag = 1;
				j = 7;						/* to exit loop */
			}
		}
	}
}

/* get_encoder()
 *
 * This is the interrupt routine for the encoder.
 * The encoder is an optical encoder knob that outputs A-B quadrature signals. The knob is
 * rotated relatively slowly, so once every 1 mS the 2 bits are polled. These are compared
 * to the previous readings and depending on the sequence, either the count value is
 * incremented, decremented or not action (NADA).
 * The logic is contained in encoder_logic[] a table.
 * Here's an example:
 *  old new action
 *  32  10
 * --------------------
 *  00  00 no action
 *  00  01 increment
 *  ...
 * The encoder state is 4 bits. The 2 LSBs are the new bits. The MSBs [3:2] are the previous readings.
 */
void get_encoder(void)
{

	uint8_t action;
	static uint8_t encoder_state;
	/* read 2 LSBs, shift up the previous bits */
	encoder_state = (encoder_state << 2) | ((GPIOB->IDR >> 10) & 0x0003); /* bits[11:10] */
	action = encoder_logic[encoder_state & 0x0F];		/* Look up action: 4 LSBs only */
	if (action == INC)
		encoder_count++;
	if (action == DEC)
		encoder_count--;
}

/* If the encoder counter is incremented, send Up volume
 * if decremented, send Down volume. Otherwise, 0
 */
uint8_t encoder_proc(void)
{
	/*	old_encoder is a global */
	uint16_t key;
	uint8_t send_flag = 0;

//	char buf1[16];

	if (encoder_count != old_encoder)
	{
		if(encoder_count > old_encoder)
		{
			key = 'V';
			send_flag = 1;
		}
		else
		{
			key = 'v';
			send_flag = 1;
		}
		/* Debug the count. Only print if the count changed */
//	  	sprintf(buf1, "Enc = %u\r\n", encoder_count);
//	  	putString(buf1);
	}
	old_encoder = encoder_count;

	if (send_flag)
	{
		return key;
	}
	else
	{
		return 0;
	}
}

/* See if key should be repeated.
 * Only those that continuously change a parameter. */
void isRepeat(uint8_t chan)
{
	uint8_t key;
	key = inKey[chan].keyCode;

	if ((key == 'V') | (key == 'v') |
		(key == 'B') | (key == 'b') |
		(key == 'T') | (key == 't') |
		(key == 'M') | (key == 'm') |
		(key == 'l') | (key == 'r'))
	{
		inKey[chan].repeat = 1;
	}
	else
	{
		inKey[chan].repeat = 0;
	}
}



