/**
  ******************************************************************************
  * @file    main.h
  * @author  D. Erickson
  * @version V0.0.1
  * @date    03/19/2011
  * @brief   Header for main.c module. Use wherever global variables are needed
  * 		 Externs for the The global variables live here
  * 		 Typedef's for global structs.
  * 		Also the main control loop. It uses the time tick to control timing.
  *
  * @history 03/19/11 dje  XPT board is working.
  * 		 First pass at the main data structure
  *
  * Still needed:
  *
  ******************************************************************************
  */

#ifndef __MAIN_H
#define __MAIN_H

/* Includes ------------------------------------------------------------------*/
#include "stm32f10x.h"
//#include "stm32_eval.h"

/* Misc #define's */
#define BLUELED GPIO_Pin_8
#define LEDPORT GPIOC

/* Preamp Defines */
#define SOURCE_MAX 7
#define ZONE_MAX 7
/* Zone timeout in seconds */
#define ZONE_TIMEOUT 20

//#define ZONE_MAX 3
#define CHAN_8_BOARD  	/* Use this define if the second quad preamp board exists */

#define VOL_MAX 	0
#define VOL_MIN 	-60
#define VOL_CHIP_MIN 47
#define VOL_STEP 	2
#define TONE_MAX 	14
#define TONE_MIN 	-14
#define TONE_STEP 	2
#define BAL_MAX 	30
#define BAL_MIN 	-30
#define BAL_STEP 	2

/* For remote keypads */
#define ADC_MIN 	1
#define ADC_MAX 	3
#define REMKEY_NUM  3

#define FPKEY 		0
#define IRKEY		1
#define UARTKEY		2
#define ENCKEY		3
#define REMKEY0		4

/* Exported types ------------------------------------------------------------*/
/* Preamp structure: one per zone. */
typedef struct
{
  uint8_t source;         	/* Input source */
  int8_t vol;         		/* Volume */
  int8_t bal;         		/* Balance */
  int8_t bass;         		/* Bass */
  int8_t mid;         		/* Midrange */
  int8_t treb;         		/* Treble */
  uint8_t mute;         	/* Mute: 0 = Normal, 1 = Mute */
  uint8_t loudness;         /* Loudness: 0 = OFF, 1 = ON */ /* Not implemented */
  uint8_t fixedGain;		/* Is this a fixed gain channel? */
}PreampTypeDef;

/* Controls structure: one per keypad. */
typedef struct
{
  uint8_t  rawCode;         /* key before LUT */
  uint8_t  rawFlag;			/* Key down from scan */
  uint8_t  keyCode;         /* Key after LUT */
  uint8_t  state;         	/* State:  0-debounce 1-repeat delay 2-repeat */
  uint8_t  sendFlag;        /* It's good. send it */
  uint16_t repeatTimer;     /* 1ms timer for key repeat */
  uint8_t  repeat;  		/* 1 if key should repeat */
  uint8_t  zone;         	/* Current zone for this keypad */
  uint16_t zoneTimer; 		/* Time until this zone resets to default */
  uint8_t  zoneDefault;		/* Zone to default to */
  uint8_t  oldKey;         	/* Previous key for filter */
}inKeyTypeDef;

/* IR Controls structure for ir.xxx */
typedef struct {
  uint16_t pulse;      		/* IR pulse width */
  uint16_t shiftReg;      	/* shift register for raw IR code */
  uint16_t debug[16];		/* pulse data for debugging */
  uint8_t  capFlag;			/* Flag from capture interrupt */
  uint16_t capCode;         /* Code from capture interrupt */
  uint16_t rawCode;         /* Key before LUT */
  uint16_t oldCode;			/* Previous key */
  uint8_t  state;         	/* Counts raw bits, 0-13 */
  uint8_t  filterState;		/* state for the filter */
  uint32_t timer;			/* FIR filter timer, incremented by 1ms int.  */
  uint32_t capture;			/* Capture the timer */
  uint32_t oldCapture;		/* Previous capture */
  uint32_t time;			/* Time difference since last capture */
}irTypeDef;


/* External / Global Variables */

/* For Time tick */
extern uint32_t ms_count;
extern uint8_t sec_flag;
extern uint16_t encoder_count;
extern uint16_t old_encoder;


uint8_t	adc_chan;					/* ADC channel counter: uses ADC numbers */
uint16_t adc_data[ADC_MAX];			/* ADC data array */
uint8_t	adc_index;					/* keypad adc counter, 0 based*/
uint8_t remkey[REMKEY_NUM];			/* remote keys are here */

/* Preamp Data Structures */
extern PreampTypeDef preamp[8];		/* Current state of the machine */
//extern PreampTypeDef  preamp_old[8];	/* Previous state */
extern uint8_t zone;				/* Current zone to be updated */

/* Keypad and IR variables */
uint16_t keypad_time;				/* Incremented by  1ms interrupt */
uint16_t ir_code;
uint16_t irTest[16];
uint16_t irFlag;					/* Global flag to main() */
uint16_t irData;


/* Function prototypes from main.c -------------------------------------------*/
void RCC_Configuration(void);
void GPIO_Configuration(void);
void ADC_Configuration(void);
void USART_Configuration(void);
void NVIC_Configuration(void);
void TIM_Configuration(void);
void DAC_Configuration(void);
void Delay(__IO uint32_t nCount);
void putString(char * str);
void putChar(int32_t ch);
void Delay_ms(__IO uint32_t nTime);
void TimingDelay_Decrement(void);
uint16_t USART_RecData(USART_TypeDef* USARTx);
uint16_t getChar(USART_TypeDef* USARTx);

/* Function prototypes from pre_control.c -----------------------------------*/
void setSource(uint8_t zone, uint8_t source);
void setZone(uint8_t chan, uint8_t zone);
void update(uint8_t zone);
void dispStruct(uint8_t zone);
void initStruct(void);
void dispatch(uint8_t chan, uint8_t cmd);
void muteOff(uint8_t zone);

/* From keypad.c ------------------------------------------------------------*/
void getKeypad(void);
uint8_t key_proc(uint8_t chan);
//uint8_t keypad_proc(void);
void ir_Test(void);
void ir_proc(void);
uint8_t remkeyProc(void);
void getRemKey(void);
void get_encoder(void);
uint8_t encoder_proc(void);
void isRepeat(uint8_t chan);


#endif /* __MAIN_H */

/****END OF FILE****/
