/**
  ******************************************************************************
  * @file    stm32f10x_it.c
  * @author  MCD Application Team
  * @version V3.3.0
  * @date    04/16/2010
  * @brief   Main Interrupt Service Routines.
  *          This file provides template for all exceptions handler and peripherals
  *          interrupt service routine.
  ******************************************************************************
  * @copy
  *
  * THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
  * WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
  * TIME. AS A RESULT, STMICROELECTRONICS SHALL NOT BE HELD LIABLE FOR ANY
  * DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
  * FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
  * CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
  *
  * <h2><center>&copy; COPYRIGHT 2010 STMicroelectronics</center></h2>
  */ 

/* Includes ------------------------------------------------------------------*/
#include "stm32f10x_conf.h"
#include "main.h"
#include "keypad.h"

/** @addtogroup STM32F10x_StdPeriph_Examples
  * @{
  */

/** @addtogroup DAC_DualModeDMA_SineWave
  * @{
  */ 

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
__IO uint16_t IC2Value = 0;
__IO uint16_t DutyCycle = 0;
__IO uint32_t Frequency = 0;

extern uint16_t remkey_limit[16];
extern const char remkey_map[8];
extern inKeyTypeDef inKey[];
extern irTypeDef ir;

/* Private function prototypes -----------------------------------------------*/
/* Private functions ---------------------------------------------------------*/

/******************************************************************************/
/*            Cortex-M3 Processor Exceptions Handlers                         */
/******************************************************************************/

/**
  * @brief  This function handles NMI exception.
  * @param  None
  * @retval None
  */
void NMI_Handler(void)
{
}

/**
  * @brief  This function handles Hard Fault exception.
  * @param  None
  * @retval None
  */
void HardFault_Handler(void)
{
  /* Go to infinite loop when Hard Fault exception occurs */
  while (1)
  {
  }
}

/**
  * @brief  This function handles Memory Manage exception.
  * @param  None
  * @retval None
  */
void MemManage_Handler(void)
{
  /* Go to infinite loop when Memory Manage exception occurs */
  while (1)
  {
  }
}

/**
  * @brief  This function handles Bus Fault exception.
  * @param  None
  * @retval None
  */
void BusFault_Handler(void)
{
  /* Go to infinite loop when Bus Fault exception occurs */
  while (1)
  {
  }
}

/**
  * @brief  This function handles Usage Fault exception.
  * @param  None
  * @retval None
  */
void UsageFault_Handler(void)
{
  /* Go to infinite loop when Usage Fault exception occurs */
  while (1)
  {
  }
}

/**
  * @brief  This function handles SVCall exception.
  * @param  None
  * @retval None
  */
void SVC_Handler(void)
{
}

/**
  * @brief  This function handles Debug Monitor exception.
  * @param  None
  * @retval None
  */
void DebugMon_Handler(void)
{
}

/**
  * @brief  This function handles PendSV_Handler exception.
  * @param  None
  * @retval None
  */
void PendSV_Handler(void)
{
}

/**
  * @brief  This function handles SysTick Handler, set to once per mS
  *
  * @param  None
  * @retval None
  *
  * Any mllisecond timers (ms_count, inKey[i].repeatTimer) are incremented here.
  *
  * When 1000 ticks happen, sec_flag is set here, cleared by main()
  *
  * The remote keyapds are polled and processed here.
  *
  * The encoder is polled and processed here.
  *
  */
void SysTick_Handler(void)
{
	uint8_t i;
//	extern int32_t irTimer;
	TimingDelay_Decrement();		/* Count 1000 ms, then set sec_flag */
	ms_count --;
	if (ms_count == 0)
	{
		ms_count = 999;
		sec_flag = 1;
	}
	/* Increment all the keypad timers */
	for(i = 0; i < (REMKEY0 + REMKEY_NUM); i++)
	{
		inKey[i].repeatTimer ++;
	}

//	irTimer ++;						/* Used to capture time of IR events */
	ir.timer ++;

	getRemKey();					/* Sample ADC for remote keypad */
	get_encoder();					/* Sample the the encoder */


}


/******************************************************************************/
/*                 STM32F10x Peripherals Interrupt Handlers                   */
/*  Add here the Interrupt Handler for the used peripheral(s) (PPP), for the  */
/*  available peripheral interrupt handler's name please refer to the startup */
/*  file (startup_stm32f10x_xx.s).                                            */
/******************************************************************************/
/**
  * @brief  This function handles TIM3 global interrupt request.
  * @param  None
  * @retval None
  */

/******************************************************************************/
/*  Timer 3 is used for the IR receiver and receives Sony IR codes.
 *  The timer is configured to measure pulse widths. It resets (triggers) on the
 *  leading edge and captures the trailing edge.
 *  IR pulses are inverted: low is leading edge, high is trailing edge.
 *  With Sony IR, there are 3 pulse widths: 2.4ms = start bit, 0.6us = 0, 1.2ms = 1
 *  The routine compares the pulse widths to limits, waits for a start bit,
 *  then shifts in accumulates 12 1's and 0's. Any errors or new start bits (bad bits, etc)
 *  cause it to reset to the beginning .
 *
 ******************************************************************************/
/**
  * @brief  TIM3 is used for the IR remote control
  * @param  None
  * @retval None
  */
void TIM3_IRQHandler(void)
{
//  static uint16_t irState;	/* Bit count. 0: Waiting for start; 1: start detected
									/*1-12 wait for 1 or 0, 13: done */
//  uint16_t pulse;
//  static uint16_t irData1;		/* This is the data shift register */
//  extern uint16_t irFlag;
//  extern uint16_t irData;
//  extern int32_t irCapture;
//  extern int32_t irTimer;

  /* Clear TIM3 Capture compare interrupt pending bit */
  TIM_ClearITPendingBit(TIM3, TIM_IT_CC2);
  /* Get the Input Capture value */
  ir.pulse =  TIM_GetCapture1(TIM3);

/* Wait for a start bit */
  if ((ir.pulse > IR_ST_MIN) && (ir.pulse < IR_ST_MAX)) 		/* Valid Start Pulse has priority */
  {
	  ir.debug[0] = ir.pulse;
	  ir.state = 1;
	  ir.shiftReg = 0;							/* Not really needed */
  }
  else
  {
	  if(ir.state != 0)
	  {
		  if ((ir.pulse > IR_HI_MIN) && (ir.pulse < IR_HI_MAX)) 	/* Valid '1' */
		  {
			  ir.shiftReg |= 0x1000 ;			/* Set Bit 12 */
			  ir.shiftReg = ir.shiftReg >> 1;	/* Shift down */
			  ir.debug[ir.state] = ir.pulse;
			  ir.state ++;
		  }
		  else if ((ir.pulse > IR_LO_MIN) && (ir.pulse < IR_LO_MAX))/* Valid '0' */
		  {
			  ir.shiftReg &= ~0x1000 ;			/* Clear Bit 12 */
			  ir.shiftReg = ir.shiftReg >> 1;	/* Shift down */
			  ir.debug[ir.state] = ir.pulse;
			  ir.state ++;
		  }
		  else /* Invalid pulse, reset state */
		  {
			  ir.state = 0;
		  }
	  }
	  else
	  {
		  ir.state = 0;
	  }

  }

  if(ir.state == 13)				/* Valid 12 bits + start bit: Send data, reset state */
  {
	ir.state   = 0;
 	ir.capCode = ir.shiftReg;
	ir.capFlag = 1;					/* Cleared by main() */

	ir.oldCapture = ir.capture;		/* Save old capture time */
	ir.capture = ir.timer;			/* Capture time of new event */
	ir.time = ir.capture - ir.oldCapture; /* Calculate the difference */
  }
}

/****END OF FILE****/
